@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 """Configuration loader
 
-$Id: ConfigLoader.py,v 1.10 2005/02/19 10:52:40 fperez Exp $"""
+$Id: ConfigLoader.py 1005 2006-01-12 08:39:26Z fperez $"""
 
 #*****************************************************************************
-#       Copyright (C) 2001-2004 Fernando Perez. <fperez@colorado.edu>
+#       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -14,12 +14,12 @@ from IPython import Release
 __author__  = '%s <%s>' % Release.authors['Fernando']
 __license__ = Release.license
 
+import exceptions
 import os
 from pprint import pprint
-import exceptions
 
 from IPython import ultraTB
-from IPython.Struct import Struct
+from IPython.ipstruct import Struct
 from IPython.genutils import *
 
 class ConfigLoaderError(exceptions.Exception):
