@@ -5,11 +5,11 @@ We define a special input line filter to allow typing lines which begin with
 '~', '/' or '.'. If one of those strings is encountered, it is automatically
 executed.
 
-$Id: InterpreterExec.py,v 1.12 2005/04/08 08:38:08 fperez Exp $"""
+$Id: InterpreterExec.py 1121 2006-02-01 21:12:20Z vivainio $"""
 
 #*****************************************************************************
 #       Copyright (C) 2004 W.J. van der Laan <gnufnork@hetdigitalegat.nl>
-#       Copyright (C) 2004 Fernando Perez <fperez@colorado.edu>
+#       Copyright (C) 2004-2006 Fernando Perez <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -20,6 +20,7 @@ __author__ = 'W.J. van der Laan <gnufnork@hetdigitalegat.nl>, '\
              '%s <%s>' %  Release.authors['Fernando']
 __license__ = Release.license
 
+# TODO: deprecated
 def prefilter_shell(self,line,continuation):
     """Alternate prefilter, modified for shell-like functionality.
 
@@ -234,26 +235,23 @@ def pysh():
 # doesn't really expose a clean API for it.  Be careful if you start making
 # many modifications here.
 
-print """\
-Welcome to pysh, a set of extensions to IPython for shell usage.
-help(pysh) -> help on the installed shell extensions and syntax.
-"""
 
 #  Set the 'cd' command to quiet mode, a more shell-like behavior
 __IPYTHON__.default_option('cd','-q')
 
+# This is redundant, ipy_user_conf.py will determine this
 # Load all of $PATH as aliases
 if os.name == 'posix':
-    # %rehash is very fast, but it doesn't check for executability, it simply
-    # dumps everything in $PATH as an alias. Use rehashx if you want more
-    # checks.
+#    # %rehash is very fast, but it doesn't check for executability, it simply
+#    # dumps everything in $PATH as an alias. Use rehashx if you want more
+#    # checks.
     __IPYTHON__.magic_rehash()
 else:
-    # Windows users: the list of extensions considered executable is read from
-    # the environment variable 'pathext'.  If this is undefined, IPython
-    # defaults to EXE, COM and BAT.
-    # %rehashx is the one which does extension analysis, at the cost of
-    # being much slower than %rehash.
+#    # Windows users: the list of extensions considered executable is read from
+#    # the environment variable 'pathext'.  If this is undefined, IPython
+#    # defaults to EXE, COM and BAT.
+#    # %rehashx is the one which does extension analysis, at the cost of
+#    # being much slower than %rehash.
     __IPYTHON__.magic_rehashx()
 
 # Remove %sc,%sx if present as aliases
@@ -264,7 +262,9 @@ __IPYTHON__.magic_unalias('sx')
 # 'gnome-terminal' are interpreted as a single alias instead of variable
 # 'gnome' minus variable 'terminal'.
 import re
-__IPYTHON__.line_split = re.compile(r'^(\s*)([\?\w\.\-\+]+\w*\s*)(\(?.*$)')
+__IPYTHON__.line_split = re.compile(r'^([\s*,;/])'
+                                    r'([\?\w\.\-\+]+\w*\s*)'
+                                    r'(\(?.*$)')
 
 # Namespace cleanup
 del re
