@@ -6,7 +6,7 @@ Uses syntax highlighting for presenting the various information elements.
 Similar in spirit to the inspect module, but all calls take a name argument to
 reference the name under which an object is being read.
 
-$Id: OInspect.py,v 1.21 2005/04/08 14:16:44 fperez Exp $
+$Id: OInspect.py 1329 2006-05-26 07:52:45Z fperez $
 """
 
 #*****************************************************************************
@@ -23,12 +23,18 @@ __license__ = Release.license
 __all__ = ['Inspector','InspectColors']
 
 # stdlib modules
-import inspect,linecache,types,StringIO,string
-
+import __builtin__
+import inspect
+import linecache
+import string
+import StringIO
+import types
+import os
 # IPython's own
-from IPython.Itpl import itpl
-from IPython.genutils import page,indent,Term
 from IPython import PyColorize
+from IPython.genutils import page,indent,Term,mkdict
+from IPython.Itpl import itpl
+from IPython.wildcard import list_namespace
 from IPython.ColorANSI import *
 
 #****************************************************************************
@@ -101,10 +107,12 @@ class myStringIO(StringIO.StringIO):
         self.write('\n')
 
 class Inspector:
-    def __init__(self,color_table,code_color_table,scheme):
+    def __init__(self,color_table,code_color_table,scheme,
+                 str_detail_level=0):
         self.color_table = color_table
         self.parser = PyColorize.Parser(code_color_table,out='str')
         self.format = self.parser.format
+        self.str_detail_level = str_detail_level
         self.set_active_scheme(scheme)
 
     def __getargspec(self,obj):
@@ -228,8 +236,17 @@ class Inspector:
             self.noinfo('file',oname)
         else:
             # run contents of file through pager starting at line
-            # where the object is defined            
-            page(self.format(open(inspect.getabsfile(obj)).read()),lineno)
+            # where the object is defined
+            ofile = inspect.getabsfile(obj)
+            
+            if (ofile.endswith('.so') or ofile.endswith('.dll')):
+                print 'File %r is binary, not printing.' % ofile
+            elif not os.path.isfile(ofile):
+                print 'File %r does not exist, not printing.' % ofile
+            else:
+                # Print only text files, not extension binaries.
+                page(self.format(open(ofile).read()),lineno)
+            #page(self.format(open(inspect.getabsfile(obj)).read()),lineno)
 
     def pinfo(self,obj,oname='',formatter=None,info=None,detail_level=0):
         """Show detailed information about an object.
@@ -262,6 +279,8 @@ class Inspector:
             ds = "Alias to the system command:\n  %s" % obj[1]
         else:
             ds = getdoc(obj)
+            if ds is None:
+                ds = '<no docstring>'
         if formatter is not None:
             ds = formatter(ds)
 
@@ -285,21 +304,22 @@ class Inspector:
         except: pass
 
         # String form, but snip if too long in ? form (full in ??)
-        try:
-            ostr = str(obj)
-            str_head = 'String Form:'
-            if not detail_level and len(ostr)>string_max:
-                ostr = ostr[:shalf] + ' <...> ' + ostr[-shalf:]
-                ostr = ("\n" + " " * len(str_head.expandtabs())).\
-                       join(map(string.strip,ostr.split("\n")))
-            if ostr.find('\n') > -1:
-                # Print multi-line strings starting at the next line.
-                str_sep = '\n'
-            else:
-                str_sep = '\t'
-            out.writeln("%s%s%s" % (header(str_head),str_sep,ostr))
-        except:
-            pass
+        if detail_level >= self.str_detail_level:
+            try:
+                ostr = str(obj)
+                str_head = 'String Form:'
+                if not detail_level and len(ostr)>string_max:
+                    ostr = ostr[:shalf] + ' <...> ' + ostr[-shalf:]
+                    ostr = ("\n" + " " * len(str_head.expandtabs())).\
+                           join(map(string.strip,ostr.split("\n")))
+                if ostr.find('\n') > -1:
+                    # Print multi-line strings starting at the next line.
+                    str_sep = '\n'
+                else:
+                    str_sep = '\t'
+                out.writeln("%s%s%s" % (header(str_head),str_sep,ostr))
+            except:
+                pass
 
         if ospace:
             out.writeln(header('Namespace:\t')+ospace)
@@ -311,12 +331,19 @@ class Inspector:
         except: pass
 
         # Filename where object was defined
+        binary_file = False
         try:
-            file = inspect.getabsfile(obj)
-            if file.endswith('<string>'):
-                file = 'Dynamically generated function. No source code available.'
-            out.writeln(header('File:\t\t')+file)
-        except: pass
+            fname = inspect.getabsfile(obj)
+            if fname.endswith('<string>'):
+                fname = 'Dynamically generated function. No source code available.'
+            if (fname.endswith('.so') or fname.endswith('.dll') or 
+                not os.path.isfile(fname)):
+                binary_file = True
+            out.writeln(header('File:\t\t')+fname)
+        except:
+            # if anything goes wrong, we don't want to show source, so it's as
+            # if the file was binary
+            binary_file = True
 
         # reconstruct the function definition and print it:
         defln = self.__getdef(obj,oname)
@@ -328,16 +355,22 @@ class Inspector:
         if ds and detail_level == 0:
                 out.writeln(header('Docstring:\n') + indent(ds))
 
+                
         # Original source code for any callable
         if detail_level:
             # Flush the source cache because inspect can return out-of-date source
             linecache.checkcache()
+            source_success = False
             try:
-                source = self.format(inspect.getsource(obj))
-                out.write(header('Source:\n')+source.rstrip())
+                if not binary_file:
+                    source = self.format(inspect.getsource(obj))
+                    out.write(header('Source:\n')+source.rstrip())
+                    source_success = True
             except:
-                if ds:
-                    out.writeln(header('Docstring:\n') + indent(ds))
+                pass
+            
+            if ds and not source_success:
+                out.writeln(header('Docstring [source file open failed]:\n') + indent(ds))
 
         # Constructor docstring for classes
         if obj_type is types.ClassType:
@@ -396,3 +429,58 @@ class Inspector:
         if output:
             page(output)
         # end pinfo
+
+    def psearch(self,pattern,ns_table,ns_search=[],
+                ignore_case=False,show_all=False):
+        """Search namespaces with wildcards for objects.
+
+        Arguments:
+
+        - pattern: string containing shell-like wildcards to use in namespace
+        searches and optionally a type specification to narrow the search to
+        objects of that type.
+
+        - ns_table: dict of name->namespaces for search.
+
+        Optional arguments:
+        
+          - ns_search: list of namespace names to include in search.
+
+          - ignore_case(False): make the search case-insensitive.
+
+          - show_all(False): show all names, including those starting with
+          underscores.
+        """
+        # defaults
+        type_pattern = 'all'
+        filter = ''
+
+        cmds = pattern.split()
+        len_cmds  =  len(cmds)
+        if len_cmds == 1:
+            # Only filter pattern given
+            filter = cmds[0]
+        elif len_cmds == 2:
+            # Both filter and type specified
+            filter,type_pattern = cmds
+        else:
+            raise ValueError('invalid argument string for psearch: <%s>' %
+                             pattern)
+
+        # filter search namespaces
+        for name in ns_search:
+            if name not in ns_table:
+                raise ValueError('invalid namespace <%s>. Valid names: %s' %
+                                 (name,ns_table.keys()))
+
+        #print 'type_pattern:',type_pattern # dbg
+        search_result = []
+        for ns_name in ns_search:
+            ns = ns_table[ns_name]
+            tmp_res = list(list_namespace(ns,type_pattern,filter,
+                                          ignore_case=ignore_case,
+                                          show_all=show_all))
+            search_result.extend(tmp_res)
+        search_result.sort()
+
+        page('\n'.join(search_result))
