@@ -2,10 +2,10 @@
 """
 Classes for handling input/output prompts.
 
-$Id: Prompts.py,v 1.25 2005/03/26 22:43:58 fperez Exp $"""
+$Id: Prompts.py 1303 2006-05-17 03:39:29Z fperez $"""
 
 #*****************************************************************************
-#       Copyright (C) 2001-2004 Fernando Perez <fperez@colorado.edu>
+#       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -19,16 +19,17 @@ __version__ = Release.version
 #****************************************************************************
 # Required modules
 import __builtin__
-import os,sys,socket
+import os
+import socket
+import sys
 import time
-from pprint import pprint,pformat
 
 # IPython's own
-from IPython.genutils import *
-from IPython.Struct import Struct
-from IPython.Magic import Macro
-from IPython.Itpl import ItplNS
 from IPython import ColorANSI
+from IPython.Itpl import ItplNS
+from IPython.ipstruct import Struct
+from IPython.macro import Macro
+from IPython.genutils import *
 
 #****************************************************************************
 #Color schemes for Prompts.
@@ -49,6 +50,7 @@ PromptColors.add_scheme(ColorANSI.ColorScheme(
 
     normal = Colors.NoColor  # color off (usu. Colors.Normal)
     ))
+
 # make some schemes as instances so we can copy them for modification easily:
 __PColLinux =  ColorANSI.ColorScheme(
     'Linux',
@@ -64,8 +66,9 @@ __PColLinux =  ColorANSI.ColorScheme(
     )
 # Don't forget to enter it into the table!
 PromptColors.add_scheme(__PColLinux)
+
 # Slightly modified Linux for light backgrounds
-__PColLightBG  = ColorANSI.ColorScheme('LightBG',**__PColLinux.colors.dict().copy())
+__PColLightBG  = __PColLinux.copy('LightBG')
 
 __PColLightBG.colors.update(
     in_prompt  = InputColors.Blue,
@@ -194,11 +197,19 @@ def str_safe(arg):
 
     If str(arg) fails, <ERROR: ... > is returned, where ... is the exception
     error message."""
-    
+
     try:
-        return str(arg)
+        out = str(arg)
+    except UnicodeError:
+        try:
+            out = arg.encode('utf_8','replace')
+        except Exception,msg:
+            # let's keep this little duplication here, so that the most common
+            # case doesn't suffer from a double try wrapping.
+            out = '<ERROR: %s>' % msg
     except Exception,msg:
-        return '<ERROR: %s>' % msg
+        out = '<ERROR: %s>' % msg
+    return out
 
 class BasePrompt:
     """Interactive prompt similar to Mathematica's."""
@@ -380,12 +391,12 @@ class CachedOutput:
     Initialize with initial and final values for cache counter (this defines
     the maximum size of the cache."""
 
-    def __init__(self,cache_size,Pprint,colors='NoColor',input_sep='\n',
-                 output_sep='\n',output_sep2='',user_ns={},
-                 ps1 = None, ps2 = None,ps_out = None,
-                 input_hist = None,pad_left=True):
+    def __init__(self,shell,cache_size,Pprint,
+                 colors='NoColor',input_sep='\n',
+                 output_sep='\n',output_sep2='',
+                 ps1 = None, ps2 = None,ps_out = None,pad_left=True):
 
-        cache_size_min = 20
+        cache_size_min = 3
         if cache_size <= 0:
             self.do_full_cache = 0
             cache_size = 0
@@ -401,9 +412,12 @@ class CachedOutput:
         self.input_sep = input_sep
 
         # we need a reference to the user-level namespace
-        self.user_ns = user_ns
+        self.shell = shell
+        self.user_ns = shell.user_ns
         # and to the user's input
-        self.input_hist = input_hist
+        self.input_hist = shell.input_hist
+        # and to the user's logger, for logging output
+        self.logger = shell.logger
 
         # Set input prompt strings and colors
         if cache_size == 0:
@@ -413,22 +427,20 @@ class CachedOutput:
         self.ps2_str = self._set_prompt_str(ps2,'   .\\D.: ','... ')
         self.ps_out_str = self._set_prompt_str(ps_out,'Out[\\#]: ','')
 
+        self.color_table = PromptColors
         self.prompt1 = Prompt1(self,sep=input_sep,prompt=self.ps1_str,
                                pad_left=pad_left)
         self.prompt2 = Prompt2(self,prompt=self.ps2_str,pad_left=pad_left)
         self.prompt_out = PromptOut(self,sep='',prompt=self.ps_out_str,
                                     pad_left=pad_left)
-        self.color_table = PromptColors
         self.set_colors(colors)
 
         # other more normal stuff
         # b/c each call to the In[] prompt raises it by 1, even the first.
         self.prompt_count = 0
-        self.cache_count = 1
         # Store the last prompt string each time, we need it for aligning
         # continuation and auto-rewrite prompts
         self.last_prompt = ''
-        self.entries = [None]  # output counter starts at 1 for the user
         self.Pprint = Pprint
         self.output_sep = output_sep
         self.output_sep2 = output_sep2
@@ -480,62 +492,63 @@ class CachedOutput:
             except KeyError:
                 pass
         if arg is not None:
+            cout_write = Term.cout.write # fast lookup
             # first handle the cache and counters
-            self.update(arg)
+
             # do not print output if input ends in ';'
             if self.input_hist[self.prompt_count].endswith(';\n'):
                 return
             # don't use print, puts an extra space
-            Term.cout.write(self.output_sep)
+            cout_write(self.output_sep)
             if self.do_full_cache:
-                Term.cout.write(str(self.prompt_out))
+                cout_write(str(self.prompt_out))
 
             if isinstance(arg,Macro):
                 print 'Executing Macro...'
                 # in case the macro takes a long time to execute
                 Term.cout.flush()
-                exec arg.value in self.user_ns
+                self.shell.runlines(arg.value)
                 return None
 
             # and now call a possibly user-defined print mechanism
-            self.display(arg)
-            Term.cout.write(self.output_sep2)
+            manipulated_val = self.display(arg)
+            
+            # user display hooks can change the variable to be stored in
+            # output history
+            
+            if manipulated_val is not None:
+                arg = manipulated_val
+                
+            # avoid recursive reference when displaying _oh/Out
+            if arg is not self.user_ns['_oh']:
+                self.update(arg)
+
+            if self.logger.log_output:
+                self.logger.log_write(repr(arg),'output')
+            cout_write(self.output_sep2)
             Term.cout.flush()
 
     def _display(self,arg):
         """Default printer method, uses pprint.
 
-        This can be over-ridden by the users to implement special formatting
-        of certain types of output."""
+        Do ip.set_hook("result_display", my_displayhook) for custom result
+        display, e.g. when your own objects need special formatting.
+        """
 
-        if self.Pprint:
-            # The following is an UGLY kludge, b/c python fails to properly
-            # identify instances of classes imported in the user namespace
-            # (they have different memory locations, I guess). Structs are
-            # essentially dicts but pprint doesn't know what to do with them.
-            try:
-                if arg.__class__.__module__ == 'Struct' and \
-                   arg.__class__.__name__ == 'Struct':
-                    out = 'Struct:\n%s' % pformat(arg.dict())
-                else:
-                    out = pformat(arg)
-            except:
-                out = pformat(arg)
-            if '\n' in out:
-                # So that multi-line strings line up with the left column of
-                # the screen, instead of having the output prompt mess up
-                # their first line.                
-                Term.cout.write('\n')
-            print >>Term.cout, out
-        else:
-            print >>Term.cout, arg
+        return self.shell.hooks.result_display(arg)
 
     # Assign the default display method:
     display = _display
 
     def update(self,arg):
         #print '***cache_count', self.cache_count # dbg
-        if self.cache_count >= self.cache_size and self.do_full_cache:
+        if len(self.user_ns['_oh']) >= self.cache_size and self.do_full_cache:
+            warn('Output cache limit (currently '+
+                  `self.cache_size`+' entries) hit.\n'
+                 'Flushing cache and resetting history counter...\n'
+                 'The only history variables available will be _,__,___ and _1\n'
+                 'with the current result.')
+
             self.flush()
         # Don't overwrite '_' and friends if '_' is in __builtin__ (otherwise
         # we cause buggy behavior for things like gettext).
@@ -548,10 +561,8 @@ class CachedOutput:
         # hackish access to top-level  namespace to create _1,_2... dynamically
         to_main = {}
         if self.do_full_cache:
-            self.cache_count += 1
-            self.entries.append(arg)
             new_result = '_'+`self.prompt_count`
-            to_main[new_result] = self.entries[-1]
+            to_main[new_result] = arg
         self.user_ns.update(to_main)
         self.user_ns['_oh'][self.prompt_count] = arg
 
@@ -559,16 +570,16 @@ class CachedOutput:
         if not self.do_full_cache:
             raise ValueError,"You shouldn't have reached the cache flush "\
                   "if full caching is not enabled!"
-        warn('Output cache limit (currently '+\
-              `self.cache_count`+' entries) hit.\n'
-             'Flushing cache and resetting history counter...\n'
-             'The only history variables available will be _,__,___ and _1\n'
-             'with the current result.')
         # delete auto-generated vars from global namespace
+        
         for n in range(1,self.prompt_count + 1):
             key = '_'+`n`
             try:
                 del self.user_ns[key]
             except: pass
-        self.prompt_count = 1
-        self.cache_count = 1
+        self.user_ns['_oh'].clear()
+        
+        if '_' not in __builtin__.__dict__:
+            self.user_ns.update({'_':None,'__':None, '___':None})
+        import gc
+        gc.collect() # xxx needed?
