@@ -4,10 +4,10 @@
 All the matplotlib support code was co-developed with John Hunter,
 matplotlib's author.
 
-$Id: Shell.py,v 1.47 2005/05/31 18:23:20 fperez Exp $"""
+$Id: Shell.py 1327 2006-05-25 03:33:58Z fperez $"""
 
 #*****************************************************************************
-#       Copyright (C) 2001-2004 Fernando Perez <fperez@colorado.edu>
+#       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -18,21 +18,22 @@ __author__  = '%s <%s>' % Release.authors['Fernando']
 __license__ = Release.license
 
 # Code begins
-import __main__
 import __builtin__
-import sys
+import __main__
+import Queue
 import os
-import code
-import threading
 import signal
+import sys
+import threading
+import time
 
 import IPython
+from IPython import ultraTB
+from IPython.genutils import Term,warn,error,flag_calls
 from IPython.iplib import InteractiveShell
 from IPython.ipmaker import make_IPython
-from IPython.genutils import Term,warn,error,flag_calls
-from IPython.Struct import Struct
 from IPython.Magic import Magic
-from IPython import ultraTB
+from IPython.ipstruct import Struct
 
 # global flag to pass around information about Ctrl-C without exceptions
 KBINT = False
@@ -48,10 +49,11 @@ USE_TK = False
 class IPShell:
     """Create an IPython instance."""
     
-    def __init__(self,argv=None,user_ns=None,debug=1,
-                 shell_class=InteractiveShell):
-        self.IP = make_IPython(argv,user_ns=user_ns,debug=debug,
-                               shell_class=shell_class)
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
+                 debug=1,shell_class=InteractiveShell):
+        self.IP = make_IPython(argv,user_ns=user_ns,
+                               user_global_ns=user_global_ns,
+                               debug=debug,shell_class=shell_class)
 
     def mainloop(self,sys_exit=0,banner=None):
         self.IP.mainloop(banner)
@@ -111,7 +113,8 @@ class IPShellEmbed:
     <cmkleffner@gmx.de> on Dec. 06/01 concerning similar uses of pyrepl, and
     by the IDL stop/continue commands."""
 
-    def __init__(self,argv=None,banner='',exit_msg=None,rc_override=None):
+    def __init__(self,argv=None,banner='',exit_msg=None,rc_override=None,
+                 user_ns=None):
         """Note that argv here is a string, NOT a list."""
         self.set_banner(banner)
         self.set_exit_msg(exit_msg)
@@ -128,14 +131,10 @@ class IPShellEmbed:
         except:
             pass # not nested with IPython
         
-        # FIXME. Passing user_ns breaks namespace handling.
-        #self.IP = make_IPython(argv,user_ns=__main__.__dict__)
-        self.IP = make_IPython(argv,rc_override=rc_override,embedded=True)
+        self.IP = make_IPython(argv,rc_override=rc_override,
+                               embedded=True,
+                               user_ns=user_ns)
 
-        self.IP.name_space_init()
-        # mark this as an embedded instance so we know if we get a crash
-        # post-mortem
-        self.IP.rc.embedded = 1
         # copy our own displayhook also
         self.sys_displayhook_embed = sys.displayhook
         # and leave the system's display hook clean
@@ -216,8 +215,8 @@ class IPShellEmbed:
 
         If you need to manually"""
 
-        if dummy not in [0,1]:
-            raise ValueError,'dummy parameter must be 0 or 1'
+        if dummy not in [0,1,False,True]:
+            raise ValueError,'dummy parameter must be boolean'
         self.__dummy_mode = dummy
 
     def get_dummy_mode(self):
@@ -265,15 +264,26 @@ class MTInteractiveShell(InteractiveShell):
     # McErlean and John Finlay.  Modified with corrections by Antoon Pardon,
     # from the pygtk mailing list, to avoid lockups with system calls.
 
+    # class attribute to indicate whether the class supports threads or not.
+    # Subclasses with thread support should override this as needed.
+    isthreaded = True
+
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
-                 user_ns = None, banner2='',**kw):
+                 user_ns=None,user_global_ns=None,banner2='',**kw):
         """Similar to the normal InteractiveShell, but with threading control"""
         
-        IPython.iplib.InteractiveShell.__init__(self,name,usage,rc,user_ns,banner2)
+        InteractiveShell.__init__(self,name,usage,rc,user_ns,
+                                  user_global_ns,banner2)
 
-        # Locking control variable
-        self.thread_ready = threading.Condition()
+        # Locking control variable.  We need to use a norma lock, not an RLock
+        # here.  I'm not exactly sure why, it seems to me like it should be
+        # the opposite, but we deadlock with an RLock.  Puzzled...
+        self.thread_ready = threading.Condition(threading.Lock())
 
+        # A queue to hold the code to be executed.  A scalar variable is NOT
+        # enough, because uses like macros cause reentrancy.
+        self.code_queue = Queue.Queue()
+        
         # Stuff to do at closing time
         self._kill = False
         on_kill = kw.get('on_kill')
@@ -310,12 +320,16 @@ class MTInteractiveShell(InteractiveShell):
             return True
 
         # Case 3
-        # Store code in self, so the execution thread can handle it
-        self.thread_ready.acquire()
-        self.code_to_run_src = source
-        self.code_to_run = code
-        self.thread_ready.wait()  # Wait until processed in timeout interval
-        self.thread_ready.release()
+        # Store code in queue, so the execution thread can handle it.
+
+        # Note that with macros and other applications, we MAY re-enter this
+        # section, so we have to acquire the lock with non-blocking semantics,
+        # else we deadlock.
+        got_lock = self.thread_ready.acquire(False)
+        self.code_queue.put(code)
+        if got_lock:
+            self.thread_ready.wait()  # Wait until processed in timeout interval
+            self.thread_ready.release()
 
         return False
 
@@ -325,7 +339,7 @@ class MTInteractiveShell(InteractiveShell):
         Multithreaded wrapper around IPython's runcode()."""
 
         # lock thread-protected stuff
-        self.thread_ready.acquire()
+        got_lock = self.thread_ready.acquire(False)
 
         # Install sigint handler
         try:
@@ -342,21 +356,31 @@ class MTInteractiveShell(InteractiveShell):
                 tokill()
             print >>Term.cout, 'Done.'
 
-        # Run pending code by calling parent class
-        if self.code_to_run is not None:
-            self.thread_ready.notify()
-            InteractiveShell.runcode(self,self.code_to_run)
+        # Flush queue of pending code by calling the run methood of the parent
+        # class with all items which may be in the queue.
+        while 1:
+            try:
+                code_to_run = self.code_queue.get_nowait()
+            except Queue.Empty:
+                break
+            if got_lock:
+                self.thread_ready.notify()
+                InteractiveShell.runcode(self,code_to_run)
+            else:
+                break
             
         # We're done with thread-protected variables
-        self.thread_ready.release()
+        if got_lock:
+            self.thread_ready.release()
         # This MUST return true for gtk threading to work
         return True
 
-    def kill (self):
+    def kill(self):
         """Kill the thread, returning when it has been shut down."""
-        self.thread_ready.acquire()
+        got_lock = self.thread_ready.acquire(False)
         self._kill = True
-        self.thread_ready.release()
+        if got_lock:
+            self.thread_ready.release()
 
 class MatplotlibShellBase:
     """Mixin class to provide the necessary modifications to regular IPython
@@ -365,8 +389,8 @@ class MatplotlibShellBase:
     Given Python's MRO, this should be used as the FIRST class in the
     inheritance hierarchy, so that it overrides the relevant methods."""
     
-    def _matplotlib_config(self,name):
-        """Return various items needed to setup the user's shell with matplotlib"""
+    def _matplotlib_config(self,name,user_ns):
+        """Return items needed to setup the user's shell with matplotlib"""
 
         # Initialize matplotlib to interactive mode always
         import matplotlib
@@ -400,15 +424,6 @@ class MatplotlibShellBase:
                 self.mpl_use._called = True
         
         self.matplotlib = matplotlib
-
-        # Take control of matplotlib's error handling, which can normally
-        # lock up the python interpreter when raw_input() is called
-        import matplotlib.backends as backend
-        backend.error_msg = error
-        
-        # we'll handle the mainloop, tell show not to
-        import matplotlib.backends
-        matplotlib.backends.show._needmain = False
         self.mpl_backend = matplotlib.rcParams['backend']
 
         # we also need to block switching of interactive backends by use()
@@ -417,38 +432,28 @@ class MatplotlibShellBase:
         # overwrite the original matplotlib.use with our wrapper
         matplotlib.use = use
 
-        # We need to detect at runtime whether show() is called by the user.
-        # For this, we wrap it into a decorator which adds a 'called' flag.
-        backend.draw_if_interactive = flag_calls(backend.draw_if_interactive)
-
         # This must be imported last in the matplotlib series, after
         # backend/interactivity choices have been made
-        try:
-            import matplotlib.pylab as pylab
-            self.pylab = pylab
-            self.pylab_name = 'pylab'
-        except ImportError:
-            import matplotlib.matlab as matlab            
-            self.pylab = matlab
-            self.pylab_name = 'matlab'
+        import matplotlib.pylab as pylab
+        self.pylab = pylab
+
+        self.pylab.show._needmain = False
+        # We need to detect at runtime whether show() is called by the user.
+        # For this, we wrap it into a decorator which adds a 'called' flag.
+        self.pylab.draw_if_interactive = flag_calls(self.pylab.draw_if_interactive)
 
         # Build a user namespace initialized with matplotlib/matlab features.
-        user_ns = {'__name__':'__main__',
-                   '__builtins__' : __builtin__ }
+        user_ns = IPython.ipapi.make_user_ns(user_ns)
 
-        # Be careful not to remove the final \n in the code string below, or
-        # things will break badly with py22 (I think it's a python bug, 2.3 is
-        # OK).
-        pname = self.pylab_name # Python can't interpolate dotted var names
         exec ("import matplotlib\n"
-              "import matplotlib.%(pname)s as %(pname)s\n"
-              "from matplotlib.%(pname)s import *\n" % locals()) in user_ns
+              "import matplotlib.pylab as pylab\n"
+              "from matplotlib.pylab import *") in user_ns
         
         # Build matplotlib info banner
         b="""
   Welcome to pylab, a matplotlib-based Python environment.
   For more information, type 'help(pylab)'.
-  """
+"""
         return user_ns,b
 
     def mplot_exec(self,fname,*where,**kw):
@@ -488,17 +493,19 @@ class MatplotlibShell(MatplotlibShellBase,InteractiveShell):
     """Single-threaded shell with matplotlib support."""
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
-                 user_ns = None, **kw):
-        user_ns,b2 = self._matplotlib_config(name)
-        InteractiveShell.__init__(self,name,usage,rc,user_ns,banner2=b2,**kw)
+                 user_ns=None,user_global_ns=None,**kw):
+        user_ns,b2 = self._matplotlib_config(name,user_ns)
+        InteractiveShell.__init__(self,name,usage,rc,user_ns,user_global_ns,
+                                  banner2=b2,**kw)
 
 class MatplotlibMTShell(MatplotlibShellBase,MTInteractiveShell):
     """Multi-threaded shell with matplotlib support."""
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
-                 user_ns = None, **kw):
-        user_ns,b2 = self._matplotlib_config(name)
-        MTInteractiveShell.__init__(self,name,usage,rc,user_ns,banner2=b2,**kw)
+                 user_ns=None,user_global_ns=None, **kw):
+        user_ns,b2 = self._matplotlib_config(name,user_ns)
+        MTInteractiveShell.__init__(self,name,usage,rc,user_ns,user_global_ns,
+                                    banner2=b2,**kw)
 
 #-----------------------------------------------------------------------------
 # Utility functions for the different GUI enabled IPShell* classes.
@@ -569,9 +576,6 @@ def hijack_gtk():
     """Modifies pyGTK's mainloop with a dummy so user code does not
     block IPython.  This function returns the original `gtk.mainloop`
     function that has been hijacked.
-
-    NOTE: Make sure you import this *AFTER* you call
-    pygtk.require(...).
     """    
     def dummy_mainloop(*args, **kw):
         pass
@@ -597,11 +601,9 @@ class IPShellGTK(threading.Thread):
     
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,debug=1,
-                 shell_class=MTInteractiveShell):
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
+                 debug=1,shell_class=MTInteractiveShell):
 
-        import pygtk
-        pygtk.require("2.0")
         import gtk
         
         self.gtk = gtk
@@ -613,18 +615,33 @@ class IPShellGTK(threading.Thread):
         if gtk.pygtk_version >= (2,4,0): mainquit = self.gtk.main_quit
         else:                            mainquit = self.gtk.mainquit
 
-        self.IP = make_IPython(argv,user_ns=user_ns,debug=debug,
+        self.IP = make_IPython(argv,user_ns=user_ns,
+                               user_global_ns=user_global_ns,
+                               debug=debug,
                                shell_class=shell_class,
                                on_kill=[mainquit])
+
+        # HACK: slot for banner in self; it will be passed to the mainloop
+        # method only and .run() needs it.  The actual value will be set by
+        # .mainloop().
+        self._banner = None 
+
         threading.Thread.__init__(self)
 
     def run(self):
-        self.IP.mainloop()
+        self.IP.mainloop(self._banner)
         self.IP.kill()
 
-    def mainloop(self):
+    def mainloop(self,sys_exit=0,banner=None):
+
+        self._banner = banner
         
-        self.gtk.timeout_add(self.TIMEOUT, self.on_timer)
+        if self.gtk.pygtk_version >= (2,4,0):
+            import gobject
+            gobject.idle_add(self.on_timer)
+        else:
+            self.gtk.idle_add(self.on_timer)
+
         if sys.platform != 'win32':
             try:
                 if self.gtk.gtk_version[0] >= 2:
@@ -648,9 +665,14 @@ class IPShellGTK(threading.Thread):
         self.join()
 
     def on_timer(self):
-        update_tk(self.tk)
-        return self.IP.runcode()
+        """Called when GTK is idle.
+
+        Must return True always, otherwise GTK stops calling it"""
         
+        update_tk(self.tk)
+        self.IP.runcode()
+        time.sleep(0.01)
+        return True
 
 class IPShellWX(threading.Thread):
     """Run a wx mainloop() in a separate thread.
@@ -661,8 +683,28 @@ class IPShellWX(threading.Thread):
     
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,debug=1,
-                 shell_class=MTInteractiveShell):
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
+                 debug=1,shell_class=MTInteractiveShell):
+
+        self.IP = make_IPython(argv,user_ns=user_ns,
+                               user_global_ns=user_global_ns,
+                               debug=debug,
+                               shell_class=shell_class,
+                               on_kill=[self.wxexit])
+
+        wantedwxversion=self.IP.rc.wxversion
+        if wantedwxversion!="0":
+            try:
+                import wxversion
+            except ImportError:
+                error('The wxversion module is needed for WX version selection')
+            else:
+                try:
+                    wxversion.select(wantedwxversion)
+                except:
+                    self.IP.InteractiveTB()
+                    error('Requested wxPython version %s could not be loaded' %
+                                                               wantedwxversion)
 
         import wxPython.wx as wx
 
@@ -673,9 +715,12 @@ class IPShellWX(threading.Thread):
         # Allows us to use both Tk and GTK.
         self.tk = get_tk()
         
-        self.IP = make_IPython(argv,user_ns=user_ns,debug=debug,
-                               shell_class=shell_class,
-                               on_kill=[self.wxexit])
+        
+        # HACK: slot for banner in self; it will be passed to the mainloop
+        # method only and .run() needs it.  The actual value will be set by
+        # .mainloop().
+        self._banner = None 
+
         self.app = None
 
     def wxexit(self, *args):
@@ -684,10 +729,12 @@ class IPShellWX(threading.Thread):
             self.app.ExitMainLoop()
 
     def run(self):
-        self.IP.mainloop()
+        self.IP.mainloop(self._banner)
         self.IP.kill()
 
-    def mainloop(self):
+    def mainloop(self,sys_exit=0,banner=None):
+
+        self._banner = banner
         
         self.start()
 
@@ -736,8 +783,8 @@ class IPShellQt(threading.Thread):
     
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,debug=0,
-                 shell_class=MTInteractiveShell):
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,
+                 debug=0,shell_class=MTInteractiveShell):
         
         import qt
 
@@ -759,20 +806,31 @@ class IPShellQt(threading.Thread):
         # Allows us to use both Tk and QT.
         self.tk = get_tk()
 
-        self.IP = make_IPython(argv,user_ns=user_ns,debug=debug,
+        self.IP = make_IPython(argv,user_ns=user_ns,
+                               user_global_ns=user_global_ns,
+                               debug=debug,
                                shell_class=shell_class,
                                on_kill=[qt.qApp.exit])
+
+        # HACK: slot for banner in self; it will be passed to the mainloop
+        # method only and .run() needs it.  The actual value will be set by
+        # .mainloop().
+        self._banner = None 
+        
         threading.Thread.__init__(self)
 
     def run(self):
-        self.IP.mainloop()
+        self.IP.mainloop(self._banner)
         self.IP.kill()
 
-    def mainloop(self):
+    def mainloop(self,sys_exit=0,banner=None):
+
+        import qt
+
+        self._banner = banner
 
-        import qt, sys
         if qt.QApplication.startingUp():
-          a = qt.QApplication.QApplication( sys.argv )
+          a = qt.QApplication.QApplication(sys.argv)
         self.timer = qt.QTimer()
         qt.QObject.connect( self.timer, qt.SIGNAL( 'timeout()' ), self.on_timer )
 
@@ -798,32 +856,36 @@ class IPShellMatplotlib(IPShell):
 
     Having this on a separate class simplifies the external driver code."""
     
-    def __init__(self,argv=None,user_ns=None,debug=1):
-        IPShell.__init__(self,argv,user_ns,debug,shell_class=MatplotlibShell)
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
+        IPShell.__init__(self,argv,user_ns,user_global_ns,debug,
+                         shell_class=MatplotlibShell)
 
 class IPShellMatplotlibGTK(IPShellGTK):
     """Subclass IPShellGTK with MatplotlibMTShell as the internal shell.
 
     Multi-threaded class, meant for the GTK* backends."""
     
-    def __init__(self,argv=None,user_ns=None,debug=1):
-        IPShellGTK.__init__(self,argv,user_ns,debug,shell_class=MatplotlibMTShell)
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
+        IPShellGTK.__init__(self,argv,user_ns,user_global_ns,debug,
+                            shell_class=MatplotlibMTShell)
 
 class IPShellMatplotlibWX(IPShellWX):
     """Subclass IPShellWX with MatplotlibMTShell as the internal shell.
 
     Multi-threaded class, meant for the WX* backends."""
     
-    def __init__(self,argv=None,user_ns=None,debug=1):
-        IPShellWX.__init__(self,argv,user_ns,debug,shell_class=MatplotlibMTShell)
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
+        IPShellWX.__init__(self,argv,user_ns,user_global_ns,debug,
+                           shell_class=MatplotlibMTShell)
 
 class IPShellMatplotlibQt(IPShellQt):
     """Subclass IPShellQt with MatplotlibMTShell as the internal shell.
 
     Multi-threaded class, meant for the Qt* backends."""
     
-    def __init__(self,argv=None,user_ns=None,debug=1):
-        IPShellQt.__init__(self,argv,user_ns,debug,shell_class=MatplotlibMTShell)
+    def __init__(self,argv=None,user_ns=None,user_global_ns=None,debug=1):
+        IPShellQt.__init__(self,argv,user_ns,user_global_ns,debug,
+                           shell_class=MatplotlibMTShell)
 
 #-----------------------------------------------------------------------------
 # Factory functions to actually start the proper thread-aware shell
@@ -853,7 +915,7 @@ def _matplotlib_shell_class():
     return sh_class
 
 # This is the one which should be called by external code.
-def start():
+def start(user_ns = None):
     """Return a running shell instance, dealing with threading options.
 
     This is a factory function which will instantiate the proper IPython shell
@@ -862,12 +924,13 @@ def start():
 
     global USE_TK
     # Crude sys.argv hack to extract the threading options.
-    if len(sys.argv) > 1:
-        if len(sys.argv) > 2:
-            arg2 = sys.argv[2]
+    argv = sys.argv
+    if len(argv) > 1:
+        if len(argv) > 2:
+            arg2 = argv[2]
             if arg2.endswith('-tk'):
                 USE_TK = True
-        arg1 = sys.argv[1]
+        arg1 = argv[1]
         if arg1.endswith('-gthread'):
             shell = IPShellGTK
         elif arg1.endswith( '-qthread' ):
@@ -880,7 +943,7 @@ def start():
             shell = IPShell
     else:
         shell = IPShell
-    return shell()
+    return shell(user_ns = user_ns)
 
 # Some aliases for backwards compatibility
 IPythonShell = IPShell
