@@ -25,12 +25,12 @@ IPython tries to:
 
   iii - serve as an embeddable, ready to go interpreter for your own programs.
 
-IPython requires Python 2.2 or newer.
+IPython requires Python 2.3 or newer.
 
-$Id: __init__.py,v 1.6 2005/03/02 07:11:15 fperez Exp $"""
+$Id: __init__.py 1328 2006-05-25 07:47:56Z fperez $"""
 
 #*****************************************************************************
-#       Copyright (C) 2001-2004 Fernando Perez. <fperez@colorado.edu>
+#       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -38,12 +38,20 @@ $Id: __init__.py,v 1.6 2005/03/02 07:11:15 fperez Exp $"""
 
 # Enforce proper version requirements
 import sys
-if sys.version[0:3] < '2.2':
-    raise ImportError, 'Python Version 2.2 or above is required.'
-        
+
+if sys.version[0:3] < '2.3':
+    raise ImportError('Python Version 2.3 or above is required for IPython.')
+
+# Make it easy to import extensions - they are always directly on pythonpath.
+# Therefore, non-IPython modules can be added to Extensions directory
+import os
+sys.path.append(os.path.dirname(__file__) + "/Extensions")
+
 # Define what gets imported with a 'from IPython import *'
-__all__ = ['deep_reload','genutils','ultraTB','DPyGetOpt','Itpl','hooks',
-           'ConfigLoader','OutputTrap','Release','Struct','Shell']
+__all__ = ['deep_reload','genutils','ipstruct','ultraTB','DPyGetOpt',
+           'Itpl','hooks','ConfigLoader','OutputTrap','Release','Shell',
+           'platutils','platutils_win32','platutils_posix','platutils_dummy',
+           'ipapi','rlineimpl']
 
 # Load __all__ in IPython namespace so that a simple 'import IPython' gives
 # access to them via IPython.<name>
@@ -53,11 +61,12 @@ for name in __all__:
 
 # Release data
 from IPython import Release # do it explicitly so pydoc can see it - pydoc bug
-__author__  = '%s <%s>\n%s <%s>\n%s <%s>' % \
-              ( Release.authors['Fernando'] + Release.authors['Janko'] + \
-                Release.authors['Nathan'] )
-__license__ = Release.license
-__version__ = Release.version
+__author__   = '%s <%s>\n%s <%s>\n%s <%s>' % \
+               ( Release.authors['Fernando'] + Release.authors['Janko'] + \
+                 Release.authors['Nathan'] )
+__license__  = Release.license
+__version__  = Release.version
+__revision__ = Release.revision
 
 # Namespace cleanup
 del name,glob,loc
