@@ -5,10 +5,10 @@ General purpose utilities.
 This is a grab-bag of stuff I find useful in most programs I write. Some of
 these things are also convenient when working at the command line.
 
-$Id: genutils.py,v 1.34 2005/03/18 09:23:48 fperez Exp $"""
+$Id: genutils.py 1349 2006-06-04 00:57:43Z fperez $"""
 
 #*****************************************************************************
-#       Copyright (C) 2001-2004 Fernando Perez. <fperez@colorado.edu>
+#       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -19,12 +19,23 @@ __author__  = '%s <%s>' % Release.authors['Fernando']
 __license__ = Release.license
 
 #****************************************************************************
-# required modules
+# required modules from the Python standard library
 import __main__
-import types,commands,time,sys,os,re,shutil
+import commands
+import os
+import re
+import shutil
+import sys
 import tempfile
+import time
+import types
+
+# Other IPython utilities
 from IPython.Itpl import Itpl,itpl,printpl
 from IPython import DPyGetOpt
+from path import path
+if os.name == "nt":
+    from IPython.winconsole import get_console_size
 
 #****************************************************************************
 # Exceptions
@@ -33,26 +44,29 @@ class Error(Exception):
     pass
 
 #----------------------------------------------------------------------------
-class Stream:
-    """Simple class to hold the various I/O streams in Term"""
-
-    def __init__(self,stream,name):
+class IOStream:
+    def __init__(self,stream,fallback):
+        if not hasattr(stream,'write') or not hasattr(stream,'flush'):
+            stream = fallback
         self.stream = stream
-        self.name = name
-        try:
-            self.fileno = stream.fileno()
-        except AttributeError:
-            msg = ("Stream <%s> looks suspicious: it lacks a 'fileno' attribute."
-                   % name)
-            print >> sys.stderr, 'WARNING:',msg
-        try:
-            self.mode = stream.mode
-        except AttributeError:
-            msg = ("Stream <%s> looks suspicious: it lacks a 'mode' attribute."
-                   % name)
-            print >> sys.stderr, 'WARNING:',msg
+        self._swrite = stream.write
+        self.flush = stream.flush
 
-class Term:
+    def write(self,data):
+        try:
+            self._swrite(data)
+        except:
+            try:
+                # print handles some unicode issues which may trip a plain
+                # write() call.  Attempt to emulate write() by using a
+                # trailing comma
+                print >> self.stream, data,
+            except:
+                # if we get here, something is seriously broken.
+                print >> sys.stderr, \
+                      'ERROR - failed to write data to stream:', self.stream
+        
+class IOTerm:
     """ Term holds the file or file-like objects for handling I/O operations.
 
     These are normally just sys.stdin, sys.stdout and sys.stderr but for
@@ -62,70 +76,20 @@ class Term:
     # In the future, having IPython channel all its I/O operations through
     # this class will make it easier to embed it into other environments which
     # are not a normal terminal (such as a GUI-based shell)
-    in_s  = Stream(sys.stdin,'cin')
-    out_s = Stream(sys.stdout,'cout')
-    err_s = Stream(sys.stderr,'cerr')
+    def __init__(self,cin=None,cout=None,cerr=None):
+        self.cin  = IOStream(cin,sys.stdin)
+        self.cout = IOStream(cout,sys.stdout)
+        self.cerr = IOStream(cerr,sys.stderr)
+        
+# Global variable to be used for all I/O
+Term = IOTerm()
 
-    # Store the three streams in (err,out,in) order so that if we need to reopen
-    # them, the error channel is reopened first to provide info.
-    streams = [err_s,out_s,in_s]
-
-    # The class globals should be the actual 'bare' streams for normal I/O to work
-    cin  = streams[2].stream
-    cout = streams[1].stream
-    cerr = streams[0].stream
+import IPython.rlineimpl as readline
+# Remake Term to use the readline i/o facilities
+if sys.platform == 'win32' and readline.have_readline:
+    
+    Term = IOTerm(cout=readline._outputfile,cerr=readline._outputfile)
     
-    def reopen_all(cls):
-        """Reopen all streams if necessary.
-
-        This should only be called if it is suspected that someting closed
-        accidentally one of the I/O streams."""
-
-        any_closed = 0
-
-        for sn in range(len(cls.streams)):
-            st = cls.streams[sn]
-            if st.stream.closed:
-                any_closed = 1
-                new_stream = os.fdopen(os.dup(st.fileno), st.mode,0)
-                cls.streams[sn] = Stream(new_stream,st.name)
-                print >> cls.streams[0].stream, \
-                      '\nWARNING:\nStream Term.%s had to be reopened!' % st.name
-
-        # Rebuild the class globals
-        cls.cin = cls.streams[2].stream
-        cls.cout = cls.streams[1].stream
-        cls.cerr = cls.streams[0].stream
-
-    reopen_all = classmethod(reopen_all)
-
-    def set_stdout(cls,stream):
-        """Set the stream """
-        cls.cout = stream
-    set_stdout = classmethod(set_stdout)
-
-    def set_stderr(cls,stream):
-        cls.cerr = stream
-    set_stderr = classmethod(set_stderr)
-
-# Windows-specific code to load Gary Bishop's readline and configure it
-# automatically for the users
-# Note: os.name on cygwin returns posix, so this should only pick up 'native'
-# windows.  Cygwin returns 'cygwin' for sys.platform.
-if os.name == 'nt':
-    try:
-        import readline
-    except ImportError:
-        pass
-    else:
-        try:
-            _out = readline.GetOutputFile()
-        except AttributeError:
-            pass
-        else:
-            Term.set_stdout(_out)
-            Term.set_stderr(_out)
-            del _out
 
 #****************************************************************************
 # Generic warning/error printer, used by everything else
@@ -168,6 +132,27 @@ def fatal(msg,exit_val=1):
 
     warn(msg,exit_val=exit_val,level=4)
 
+#---------------------------------------------------------------------------
+# Debugging routines
+# 
+def debugx(expr,pre_msg=''):
+    """Print the value of an expression from the caller's frame.
+    
+    Takes an expression, evaluates it in the caller's frame and prints both
+    the given expression and the resulting value (as well as a debug mark
+    indicating the name of the calling function.  The input must be of a form
+    suitable for eval().
+
+    An optional message can be passed, which will be prepended to the printed
+    expr->value pair."""
+    
+    cf = sys._getframe(1)
+    print '[DBG:%s] %s%s -> %r' % (cf.f_code.co_name,pre_msg,expr,
+                                   eval(expr,cf.f_globals,cf.f_locals))
+
+# deactivate it by uncommenting the following line, which makes it a no-op
+#def debugx(expr,pre_msg=''): pass
+
 #----------------------------------------------------------------------------
 StringTypes = types.StringTypes
 
@@ -269,6 +254,10 @@ def system(cmd,verbose=0,debug=0,header=''):
     if not debug: stat = os.system(cmd)
     return stat
 
+# This function is used by ipython in a lot of places to make system calls.
+# We need it to be slightly different under win32, due to the vagaries of
+# 'network shares'.  A win32 override is below.
+
 def shell(cmd,verbose=0,debug=0,header=''):
     """Execute a command in the system shell, always return None.
 
@@ -292,6 +281,27 @@ def shell(cmd,verbose=0,debug=0,header=''):
     if not debug:
         os.system(cmd)
 
+# override shell() for win32 to deal with network shares
+if os.name in ('nt','dos'):
+
+    shell_ori = shell
+
+    def shell(cmd,verbose=0,debug=0,header=''):
+        if os.getcwd().startswith(r"\\"):
+            path = os.getcwd()
+            # change to c drive (cannot be on UNC-share when issuing os.system,
+            # as cmd.exe cannot handle UNC addresses)
+            os.chdir("c:")
+            # issue pushd to the UNC-share and then run the command
+            try:
+                shell_ori('"pushd %s&&"'%path+cmd,verbose,debug,header)
+            finally:
+                os.chdir(path)
+        else:
+            shell_ori(cmd,verbose,debug,header)
+
+    shell.__doc__ = shell_ori.__doc__
+
 def getoutput(cmd,verbose=0,debug=0,header='',split=0):
     """Dummy substitute for perl's backquotes.
 
@@ -302,11 +312,19 @@ def getoutput(cmd,verbose=0,debug=0,header='',split=0):
     - split(0): if true, the output is returned as a list split on newlines.
 
     Note: a stateful version of this function is available through the
-    SystemExec class."""
+    SystemExec class.
+    
+    This is pretty much deprecated and rarely used, 
+    genutils.getoutputerror may be what you need.
+    
+    """
 
     if verbose or debug: print header+cmd
     if not debug:
-        output = commands.getoutput(cmd)
+        output = os.popen(cmd).read()
+        # stipping last \n is here for backwards compat.
+        if output.endswith('\n'):
+            output = output[:-1]
         if split:
             return output.split('\n')
         else:
@@ -410,6 +428,21 @@ def mutex_opts(dict,ex_op):
             raise ValueError,'\n*** ERROR in Arguments *** '\
                   'Options '+op1+' and '+op2+' are mutually exclusive.'
 
+#-----------------------------------------------------------------------------
+def get_py_filename(name):
+    """Return a valid python filename in the current directory.
+
+    If the given name is not a file, it adds '.py' and searches again.
+    Raises IOError with an informative message if the file isn't found."""
+
+    name = os.path.expanduser(name)
+    if not os.path.isfile(name) and not name.endswith('.py'):
+        name += '.py'
+    if os.path.isfile(name):
+        return name
+    else:
+        raise IOError,'File `%s` not found.' % name
+
 #-----------------------------------------------------------------------------
 def filefind(fname,alt_dirs = None):
     """Return the given filename either in the current directory, if it
@@ -438,6 +471,21 @@ def filefind(fname,alt_dirs = None):
     raise IOError,'File' + `fname` + \
           ' not found in current or supplied directories:' + `alt_dirs`
 
+#----------------------------------------------------------------------------
+def file_read(filename):
+    """Read a file and close it.  Returns the file source."""
+    fobj = open(filename,'r');
+    source = fobj.read();
+    fobj.close()
+    return source
+
+def file_readlines(filename):
+    """Read a file and close it.  Returns the file source using readlines()."""
+    fobj = open(filename,'r');
+    lines = fobj.readlines();
+    fobj.close()
+    return lines
+
 #----------------------------------------------------------------------------
 def target_outdated(target,deps):
     """Determine whether a target is out of date.
@@ -711,17 +759,29 @@ def get_home_dir():
     We first try $HOME.  Absent that, on NT it's $HOMEDRIVE\$HOMEPATH.
 
     Currently only Posix and NT are implemented, a HomeDirError exception is
-    raised for all other OSes. """ #'
+    raised for all other OSes. """
 
+    isdir = os.path.isdir
+    env = os.environ
     try:
-        return os.environ['HOME']
+        homedir = env['HOME']
+        if not isdir(homedir):
+            # in case a user stuck some string which does NOT resolve to a
+            # valid path, it's as good as if we hadn't foud it
+            raise KeyError
+        return homedir
     except KeyError:
         if os.name == 'posix':
             raise HomeDirError,'undefined $HOME, IPython can not proceed.'
         elif os.name == 'nt':
             # For some strange reason, win9x returns 'nt' for os.name.
             try:
-                return os.path.join(os.environ['HOMEDRIVE'],os.environ['HOMEPATH'])
+                homedir = os.path.join(env['HOMEDRIVE'],env['HOMEPATH'])
+                if not isdir(homedir):
+                    homedir = os.path.join(env['USERPROFILE'])
+                    if not isdir(homedir):
+                        raise HomeDirError
+                return homedir
             except:
                 try:
                     # Use the registry to get the 'My Documents' folder.
@@ -730,7 +790,16 @@ def get_home_dir():
                                        "Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders")
                     homedir = wreg.QueryValueEx(key,'Personal')[0]
                     key.Close()
+                    if not isdir(homedir):
+                        e = ('Invalid "Personal" folder registry key '
+                             'typically "My Documents".\n'
+                             'Value: %s\n'
+                             'This is not a valid directory on your system.' %
+                             homedir)
+                        raise HomeDirError(e)
                     return homedir
+                except HomeDirError:
+                    raise
                 except:
                     return 'C:\\'
         elif os.name == 'dos':
@@ -780,6 +849,17 @@ class LSString(str):
 
     n = nlstr = property(get_nlstr)
 
+    def get_paths(self):
+        try:
+            return self.__paths
+        except AttributeError:
+            self.__paths = [path(p) for p in self.split('\n') if os.path.exists(p)]
+            return self.__paths
+    
+    p = paths = property(get_paths)
+
+
+#----------------------------------------------------------------------------
 class SList(list):
     """List derivative with a special access attributes.
 
@@ -814,7 +894,57 @@ class SList(list):
             return self.__nlstr
 
     n = nlstr = property(get_nlstr)
+    
+    def get_paths(self):
+        try:
+            return self.__paths
+        except AttributeError:
+            self.__paths = [path(p) for p in self if os.path.exists(p)]
+            return self.__paths
+    
+    p = paths = property(get_paths)
 
+#----------------------------------------------------------------------------
+def esc_quotes(strng):
+    """Return the input string with single and double quotes escaped out"""
+
+    return strng.replace('"','\\"').replace("'","\\'")
+
+#----------------------------------------------------------------------------
+def make_quoted_expr(s):
+    """Return string s in appropriate quotes, using raw string if possible.
+    
+    Effectively this turns string: cd \ao\ao\
+    to: r"cd \ao\ao\_"[:-1]
+    
+    Note the use of raw string and padding at the end to allow trailing backslash.
+    
+    """
+    
+    tail = ''
+    tailpadding = ''
+    raw  = ''
+    if "\\" in s:
+        raw = 'r'
+        if s.endswith('\\'):
+            tail = '[:-1]'
+            tailpadding = '_'
+    if '"' not in s:
+        quote = '"'
+    elif "'" not in s:
+        quote = "'"
+    elif '"""' not in s and not s.endswith('"'):
+        quote = '"""'
+    elif "'''" not in s and not s.endswith("'"):
+        quote = "'''"
+    else:
+        # give up, backslash-escaped string will do
+        return '"%s"' % esc_quotes(s)
+    res = itpl("$raw$quote$s$tailpadding$quote$tail")
+    return res
+
+
+#----------------------------------------------------------------------------
 def raw_input_multi(header='', ps1='==> ', ps2='..> ',terminate_str = '.'):
     """Take multiple lines of input.
 
@@ -862,31 +992,40 @@ def ask_yes_no(prompt,default=None):
 
     If default is given (one of 'y','n'), it is used if the user input is
     empty. Otherwise the question is repeated until an answer is given.
-    If EOF occurs 20 times consecutively, the default answer is assumed,
-    or if there is no default, an exception is raised to prevent infinite
-    loops.
+
+    An EOF is treated as the default answer.  If there is no default, an
+    exception is raised to prevent infinite loops.
 
     Valid answers are: y/yes/n/no (match is not case sensitive)."""
 
-    answers = {'y':1,'n':0,'yes':1,'no':0}
+    answers = {'y':True,'n':False,'yes':True,'no':False}
     ans = None
-    eofs, max_eofs = 0, 20
     while ans not in answers.keys():
         try:
             ans = raw_input(prompt+' ').lower()
             if not ans:  # response was an empty string
                 ans = default
-            eofs = 0
-        except (EOFError,KeyboardInterrupt):
-            eofs = eofs + 1
-            if eofs >= max_eofs:
-                if default in answers.keys():
-                    ans = default
-                else:
-                    raise
+        except KeyboardInterrupt:
+            pass
+        except EOFError:
+            if default in answers.keys():
+                ans = default
+                print
+            else:
+                raise
             
     return answers[ans]
 
+#----------------------------------------------------------------------------
+def marquee(txt='',width=78,mark='*'):
+    """Return the input string centered in a 'marquee'."""
+    if not txt:
+        return (mark*width)[:width]
+    nmark = (width-len(txt)-2)/len(mark)/2
+    if nmark < 0: nmark =0
+    marks = mark*nmark
+    return '%s %s %s' % (marks,txt,marks)
+
 #----------------------------------------------------------------------------
 class EvalDict:
     """
@@ -936,6 +1075,19 @@ def qwflat(words,sep=None,maxsplit=-1):
     """Calls qw(words) in flat mode. It's just a convenient shorthand."""
     return qw(words,1,sep,maxsplit)
 
+#----------------------------------------------------------------------------
+def qw_lol(indata):
+    """qw_lol('a b') -> [['a','b']],
+    otherwise it's just a call to qw().
+
+    We need this to make sure the modules_some keys *always* end up as a
+    list of lists."""
+
+    if type(indata) in StringTypes:
+        return [qw(indata)]
+    else:
+        return qw(indata)
+
 #-----------------------------------------------------------------------------
 def list_strings(arg):
     """Always return a list of strings, given a string or list of strings
@@ -1062,6 +1214,31 @@ def get_pager_start(pager,start):
     return start_string
 
 #----------------------------------------------------------------------------
+if os.name == "nt":
+    import msvcrt
+    def page_more():
+        """ Smart pausing between pages
+
+        @return:    True if need print more lines, False if quit
+        """
+        Term.cout.write('---Return to continue, q to quit--- ')
+        ans = msvcrt.getch()
+        if ans in ("q", "Q"):
+            result = False
+        else:
+            result = True
+        Term.cout.write("\b"*37 + " "*37 + "\b"*37)
+        return result
+else:
+    def page_more():
+        ans = raw_input('---Return to continue, q to quit--- ')
+        if ans.lower().startswith('q'):
+            return False
+        else:
+            return True
+
+esc_re = re.compile(r"(\x1b[^m]+m)")
+
 def page_dumb(strng,start=0,screen_lines=25):
     """Very dumb 'pager' in Python, for when nothing else works.
 
@@ -1073,12 +1250,16 @@ def page_dumb(strng,start=0,screen_lines=25):
     if len(screens) == 1:
         print >>Term.cout, os.linesep.join(screens[0])
     else:
+        last_escape = ""
         for scr in screens[0:-1]:
-            print >>Term.cout, os.linesep.join(scr)
-            ans = raw_input('---Return to continue, q to quit--- ')
-            if ans.lower().startswith('q'):
+            hunk = os.linesep.join(scr)
+            print >>Term.cout, last_escape + hunk
+            if not page_more():
                 return
-        print >>Term.cout, os.linesep.join(screens[-1])
+            esc_list = esc_re.findall(hunk)
+            if len(esc_list) > 0:
+                last_escape = esc_list[-1]
+        print >>Term.cout, last_escape + os.linesep.join(screens[-1])
 
 #----------------------------------------------------------------------------
 def page(strng,start=0,screen_lines=0,pager_cmd = None):
@@ -1118,7 +1299,10 @@ def page(strng,start=0,screen_lines=0,pager_cmd = None):
     # terminals. If someone later feels like refining it, it's not hard.
     numlines = max(num_newlines,int(len_str/80)+1)
 
-    screen_lines_def = 25 # default value if we can't auto-determine
+    if os.name == "nt":
+        screen_lines_def = get_console_size(defaulty=25)[1]
+    else:
+        screen_lines_def = 25 # default value if we can't auto-determine
 
     # auto-determine screen size
     if screen_lines <= 0:
@@ -1374,16 +1558,6 @@ def list2dict2(lst,default=''):
 def flatten(seq):
     """Flatten a list of lists (NOT recursive, only works for 2d lists)."""
 
-    # bug in python??? (YES. Fixed in 2.2, let's leave the kludgy fix in).
-
-    # if the x=0 isn't made, a *global* variable x is left over after calling
-    # this function, with the value of the last element in the return
-    # list. This does seem like a bug big time to me.
-
-    # the problem is fixed with the x=0, which seems to force the creation of
-    # a local name
-
-    x = 0 
     return [x for subseq in seq for x in subseq]
 
 #----------------------------------------------------------------------------
@@ -1401,17 +1575,22 @@ def chop(seq,size):
     return map(chunk,xrange(0,len(seq),size))
 
 #----------------------------------------------------------------------------
-def with(object, **args):
+# with is a keyword as of python 2.5, so this function is renamed to withobj
+# from its old 'with' name.
+def with_obj(object, **args):
     """Set multiple attributes for an object, similar to Pascal's with.
 
     Example:
-    with(jim,
-         born = 1960,
-         haircolour = 'Brown',
-         eyecolour = 'Green')
+    with_obj(jim,
+             born = 1960,
+             haircolour = 'Brown',
+             eyecolour = 'Green')
 
     Credit: Greg Ewing, in
-    http://mail.python.org/pipermail/python-list/2001-May/040703.html"""
+    http://mail.python.org/pipermail/python-list/2001-May/040703.html.
+
+    NOTE: up until IPython 0.7.2, this was called simply 'with', but 'with'
+    has become a keyword for Python 2.5, so we had to rename it."""
 
     object.__dict__.update(args)
 
@@ -1494,6 +1673,15 @@ def map_method(method,object_list,*argseq,**kw):
         idx += 1
     return out_list
 
+#----------------------------------------------------------------------------
+def import_fail_info(mod_name,fns=None):
+    """Inform load failure for a module."""
+
+    if fns == None:
+        warn("Loading of %s failed.\n" % (mod_name,))
+    else:
+        warn("Loading of %s from %s failed.\n" % (fns,mod_name))
+
 #----------------------------------------------------------------------------
 # Proposed popitem() extension, written as a method
 
