@@ -6,10 +6,10 @@ Requires Python 2.1 or better.
 
 This file contains the main make_IPython() starter function.
 
-$Id: ipmaker.py,v 1.33 2005/05/13 21:20:00 fperez Exp $"""
+$Id: ipmaker.py 1347 2006-06-04 00:42:44Z fperez $"""
 
 #*****************************************************************************
-#       Copyright (C) 2001-2004 Fernando Perez. <fperez@colorado.edu>
+#       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
 #
 #  Distributed under the terms of the BSD License.  The full license is in
 #  the file COPYING, distributed as part of this software.
@@ -36,23 +36,27 @@ All Rights Reserved."""
 # Required modules
 
 # From the standard library
-import __main__, __builtin__
-import os,sys,types,re
+import __main__
+import __builtin__
+import os
+import re
+import sys
+import types
 from pprint import pprint,pformat
 
 # Our own
 from IPython import DPyGetOpt
-from IPython.Struct import Struct
+from IPython.ipstruct import Struct
 from IPython.OutputTrap import OutputTrap
 from IPython.ConfigLoader import ConfigLoader
-from IPython.iplib import InteractiveShell,qw_lol,import_fail_info
+from IPython.iplib import InteractiveShell
 from IPython.usage import cmd_line_usage,interactive_usage
-from IPython.Prompts import CachedOutput
 from IPython.genutils import *
 
 #-----------------------------------------------------------------------------
-def make_IPython(argv=None,user_ns=None,debug=1,rc_override=None,
-                 shell_class=InteractiveShell,embedded=False,**kw):
+def make_IPython(argv=None,user_ns=None,user_global_ns=None,debug=1,
+                 rc_override=None,shell_class=InteractiveShell,
+                 embedded=False,**kw):
     """This is a dump of IPython into a single function.
 
     Later it will have to be broken up in a sensible manner.
@@ -82,32 +86,18 @@ def make_IPython(argv=None,user_ns=None,debug=1,rc_override=None,
     # __IP.name. We set its name via the first parameter passed to
     # InteractiveShell:
 
-    IP = shell_class('__IP',user_ns=user_ns,**kw)
+    IP = shell_class('__IP',user_ns=user_ns,user_global_ns=user_global_ns,
+                     embedded=embedded,**kw)
 
     # Put 'help' in the user namespace
-    try:
-        from site import _Helper
-    except ImportError:
-        # Use the _Helper class from Python 2.2 for older Python versions
-        class _Helper:
-            def __repr__(self):
-                return "Type help() for interactive help, " \
-                       "or help(object) for help about object."
-            def __call__(self, *args, **kwds):
-                import pydoc
-                return pydoc.help(*args, **kwds)
-    else:
-        IP.user_ns['help'] = _Helper()
+    from site import _Helper
+    IP.user_ns['help'] = _Helper()
+
 
     if DEVDEBUG:
         # For developer debugging only (global flag)
         from IPython import ultraTB
         sys.excepthook = ultraTB.VerboseTB(call_pdb=1)
-    else:
-        # IPython itself shouldn't crash. This will produce a detailed
-        # post-mortem if it does
-        from IPython import CrashHandler
-        sys.excepthook = CrashHandler.CrashHandler(IP)
 
     IP.BANNER_PARTS = ['Python %s\n'
                          'Type "copyright", "credits" or "license" '
@@ -123,7 +113,9 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
 
     IP.usage = interactive_usage
 
-    # Platform-dependent suffix and directory names
+    # Platform-dependent suffix and directory names.  We use _ipython instead
+    # of .ipython under win32 b/c there's software that breaks with .named
+    # directories on that platform.
     if os.name == 'posix':
         rc_suffix = ''
         ipdir_def = '.ipython'
@@ -132,9 +124,11 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
         ipdir_def = '_ipython'
 
     # default directory for configuration
-    ipythondir = os.path.abspath(os.environ.get('IPYTHONDIR',
+    ipythondir_def = os.path.abspath(os.environ.get('IPYTHONDIR',
                                  os.path.join(IP.home_dir,ipdir_def)))
 
+    sys.path.insert(0, '') # add . to sys.path. Fix from Prabhu Ramachandran
+
     # we need the directory where IPython itself is installed
     import IPython
     IPython_dir = os.path.dirname(IPython.__file__)
@@ -155,18 +149,21 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # DPyGetOpt syntax (=s,=i,:f,etc).
 
     # Make sure there's a space before each end of line (they get auto-joined!)
-    cmdline_opts = ('autocall! autoindent! automagic! banner! cache_size|cs=i '
+    cmdline_opts = ('autocall=i autoindent! automagic! banner! cache_size|cs=i '
                     'c=s classic|cl color_info! colors=s confirm_exit! '
-                    'debug! deep_reload! editor=s log|l messages! nosep pdb! '
+                    'debug! deep_reload! editor=s log|l messages! nosep '
+                    'object_info_string_level=i pdb! '
                     'pprint! prompt_in1|pi1=s prompt_in2|pi2=s prompt_out|po=s '
                     'quick screen_length|sl=i prompts_pad_left=i '
                     'logfile|lf=s logplay|lp=s profile|p=s '
                     'readline! readline_merge_completions! '
                     'readline_omit__names! '
                     'rcfile=s separate_in|si=s separate_out|so=s '
-                    'separate_out2|so2=s xmode=s '
+                    'separate_out2|so2=s xmode=s wildcards_case_sensitive! '
                     'magic_docstrings system_verbose! '
-                    'multi_line_specials!')
+                    'multi_line_specials! '
+                    'wxversion=s '
+                    'autoedit_syntax!')
 
     # Options that can *only* appear at the cmd line (not in rcfiles).
     
@@ -181,7 +178,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # Set sensible command line defaults.
     # This should have everything from  cmdline_opts and cmdline_only
     opts_def = Struct(autocall = 1,
-                      autoindent=0,
+                      autoedit_syntax = 0,
+                      autoindent = 0,
                       automagic = 1,
                       banner = 1,
                       cache_size = 1000,
@@ -195,19 +193,20 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       editor = '0',
                       help = 0,
                       ignore = 0,
-                      ipythondir = ipythondir,
+                      ipythondir = ipythondir_def,
                       log = 0,
                       logfile = '',
                       logplay = '',
                       multi_line_specials = 1,
                       messages = 1,
+                      object_info_string_level = 0,
                       nosep = 0,
                       pdb = 0,
                       pprint = 0,
                       profile = '',
-                      prompt_in1 = 'In [\\#]:',
-                      prompt_in2 = '   .\\D.:',
-                      prompt_out = 'Out[\\#]:',
+                      prompt_in1 = 'In [\\#]: ',
+                      prompt_in2 = '   .\\D.: ',
+                      prompt_out = 'Out[\\#]: ',
                       prompts_pad_left = 1,
                       quick = 0,
                       readline = 1,
@@ -227,6 +226,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       upgrade = 0,
                       Version = 0,
                       xmode = 'Verbose',
+                      wildcards_case_sensitive = 1,
+                      wxversion = '0',
                       magic_docstrings = 0,  # undocumented, for doc generation
                       )
     
@@ -290,7 +291,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     except:
         print cmd_line_usage
         warn('\nError in Arguments: ' + `sys.exc_value`)
-        sys.exit()
+        sys.exit(1)
 
     # convert the options dict to a struct for much lighter syntax later
     opts = Struct(getopt.optionValues)
@@ -313,6 +314,10 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
         IP.magic_magic('-latex')
         sys.exit()
 
+    # add personal ipythondir to sys.path so that users can put things in
+    # there for customization
+    sys.path.append(os.path.abspath(opts_all.ipythondir))
+
     # Create user config directory if it doesn't exist. This must be done
     # *after* getting the cmd line options.
     if not os.path.isdir(opts_all.ipythondir):
@@ -326,9 +331,6 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     mutex_opts(opts,[qw('log logfile'),qw('rcfile profile'),
                      qw('classic profile'),qw('classic rcfile')])
 
-    # default logfilename used when -log is called.
-    IP.LOGDEF = 'ipython.log'
-
     #---------------------------------------------------------------------------
     # Log replay
     
@@ -411,6 +413,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
 
     # 'profiles' are a shorthand notation for config filenames
     if opts_all.profile:
+            
         try:
             opts_all.rcfile = filefind('ipythonrc-' + opts_all.profile
                                        + rc_suffix,
@@ -418,8 +421,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
         except IOError:
            if opts_all.debug:  IP.InteractiveTB()
            opts.profile = ''  # remove profile from options if invalid
-           warn('Profile configuration file %s not found. Ignoring request.'
-                % (opts_all.profile) )
+           # We won't warn anymore, primary method is ipy_profile_PROFNAME
+           # which does trigger a warning.
 
     # load the config file
     rcfiledata = None
@@ -453,74 +456,74 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     #------------------------------------------------------------------------
     # Execute user config
 
-    # first, create a valid config structure with the right precedence order:
-    # defaults < rcfile < command line
+    # Create a valid config structure with the right precedence order:
+    # defaults < rcfile < command line.  This needs to be in the instance, so
+    # that method calls below that rely on it find it.
     IP.rc = rc_def.copy()
-    IP.rc.update(opts_def)
+
+    # Work with a local alias inside this routine to avoid unnecessary
+    # attribute lookups.
+    IP_rc = IP.rc
+    
+    IP_rc.update(opts_def)
     if rcfiledata:
         # now we can update 
-        IP.rc.update(rcfiledata)
-    IP.rc.update(opts)
-    IP.rc.update(rc_override)
+        IP_rc.update(rcfiledata)
+    IP_rc.update(opts)
+    IP_rc.update(rc_override)
 
     # Store the original cmd line for reference:
-    IP.rc.opts = opts
-    IP.rc.args = args
+    IP_rc.opts = opts
+    IP_rc.args = args
 
     # create a *runtime* Struct like rc for holding parameters which may be
     # created and/or modified by runtime user extensions.
     IP.runtime_rc = Struct()
 
-    # from this point on, all config should be handled through IP.rc,
+    # from this point on, all config should be handled through IP_rc,
     # opts* shouldn't be used anymore.
 
-    # add personal .ipython dir to sys.path so that users can put things in
-    # there for customization
-    sys.path.append(IP.rc.ipythondir)
-    sys.path.insert(0, '') # add . to sys.path. Fix from Prabhu Ramachandran
     
-    # update IP.rc with some special things that need manual
+    # update IP_rc with some special things that need manual
     # tweaks. Basically options which affect other options. I guess this
     # should just be written so that options are fully orthogonal and we
     # wouldn't worry about this stuff!
 
-    if IP.rc.classic:
-        IP.rc.quick = 1
-        IP.rc.cache_size = 0
-        IP.rc.pprint = 0
-        IP.rc.prompt_in1 = '>>> '
-        IP.rc.prompt_in2 = '... '
-        IP.rc.prompt_out = ''
-        IP.rc.separate_in = IP.rc.separate_out = IP.rc.separate_out2 = '0'
-        IP.rc.colors = 'NoColor'
-        IP.rc.xmode = 'Plain'
+    if IP_rc.classic:
+        IP_rc.quick = 1
+        IP_rc.cache_size = 0
+        IP_rc.pprint = 0
+        IP_rc.prompt_in1 = '>>> '
+        IP_rc.prompt_in2 = '... '
+        IP_rc.prompt_out = ''
+        IP_rc.separate_in = IP_rc.separate_out = IP_rc.separate_out2 = '0'
+        IP_rc.colors = 'NoColor'
+        IP_rc.xmode = 'Plain'
 
+    IP.pre_config_initialization()
     # configure readline
     # Define the history file for saving commands in between sessions
-    if IP.rc.profile:
-        histfname = 'history-%s' % IP.rc.profile
+    if IP_rc.profile:
+        histfname = 'history-%s' % IP_rc.profile
     else:
         histfname = 'history'
     IP.histfile = os.path.join(opts_all.ipythondir,histfname)
-    # Load readline proper
-    if IP.rc.readline:
-        IP.init_readline()
 
     # update exception handlers with rc file status
     otrap.trap_out()  # I don't want these messages ever.
-    IP.magic_xmode(IP.rc.xmode)
+    IP.magic_xmode(IP_rc.xmode)
     otrap.release_out()
 
     # activate logging if requested and not reloading a log
-    if IP.rc.logplay:
-        IP.magic_logstart(IP.rc.logplay + ' append')
-    elif  IP.rc.logfile:
-        IP.magic_logstart(IP.rc.logfile)
-    elif IP.rc.log:
+    if IP_rc.logplay:
+        IP.magic_logstart(IP_rc.logplay + ' append')
+    elif  IP_rc.logfile:
+        IP.magic_logstart(IP_rc.logfile)
+    elif IP_rc.log:
         IP.magic_logstart()
 
     # find user editor so that it we don't have to look it up constantly
-    if IP.rc.editor.strip()=='0':
+    if IP_rc.editor.strip()=='0':
         try:
             ed = os.environ['EDITOR']
         except KeyError:
@@ -528,12 +531,16 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                 ed = 'vi'  # the only one guaranteed to be there!
             else:
                 ed = 'notepad' # same in Windows!
-        IP.rc.editor = ed
+        IP_rc.editor = ed
+
+    # Keep track of whether this is an embedded instance or not (useful for
+    # post-mortems).
+    IP_rc.embedded = IP.embedded
 
     # Recursive reload
     try:
         from IPython import deep_reload
-        if IP.rc.deep_reload:
+        if IP_rc.deep_reload:
             __builtin__.reload = deep_reload.reload
         else:
             __builtin__.dreload = deep_reload.reload
@@ -547,25 +554,25 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # defining things on the command line, and %who works as expected.
 
     # DON'T do anything that affects the namespace beyond this point!
-    IP.internal_ns = __main__.__dict__.copy()
+    IP.internal_ns.update(__main__.__dict__)
 
     #IP.internal_ns.update(locals()) # so our stuff doesn't show up in %who
 
     # Now run through the different sections of the users's config
-    if IP.rc.debug:
+    if IP_rc.debug:    
         print 'Trying to execute the following configuration structure:'
         print '(Things listed first are deeper in the inclusion tree and get'
         print 'loaded first).\n'
-        pprint(IP.rc.__dict__)
+        pprint(IP_rc.__dict__)
         
-    for mod in IP.rc.import_mod:
+    for mod in IP_rc.import_mod:
         try:
             exec 'import '+mod in IP.user_ns
         except :
             IP.InteractiveTB()
             import_fail_info(mod)
 
-    for mod_fn in IP.rc.import_some:
+    for mod_fn in IP_rc.import_some:
         if mod_fn == []: break
         mod,fn = mod_fn[0],','.join(mod_fn[1:])
         try:
@@ -574,14 +581,14 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
             IP.InteractiveTB()
             import_fail_info(mod,fn)
 
-    for mod in IP.rc.import_all:
+    for mod in IP_rc.import_all:
         try:
             exec 'from '+mod+' import *' in IP.user_ns
         except :
             IP.InteractiveTB()
             import_fail_info(mod)
 
-    for code in IP.rc.execute:
+    for code in IP_rc.execute:
         try:
             exec code in IP.user_ns
         except:
@@ -589,7 +596,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
             warn('Failure executing code: ' + `code`)
 
     # Execute the files the user wants in ipythonrc
-    for file in IP.rc.execfile:
+    for file in IP_rc.execfile:
         try:
             file = filefind(file,sys.path+[IPython_dir])
         except IOError:
@@ -597,13 +604,46 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
         else:
             IP.safe_execfile(os.path.expanduser(file),IP.user_ns)
 
-    # Load user aliases
-    for alias in IP.rc.alias:
-        IP.magic_alias(alias)
-
+    # finally, try importing ipy_*_conf for final configuration
+    try:
+        import ipy_system_conf
+    except ImportError:
+        if opts_all.debug:  IP.InteractiveTB()
+        warn("Could not import 'ipy_system_conf'")        
+    except:
+        IP.InteractiveTB()
+        import_fail_info('ipy_system_conf')
+        
+    if opts_all.profile:
+        profmodname = 'ipy_profile_' + opts_all.profile
+        try:
+            __import__(profmodname)
+        except ImportError:
+            # only warn if ipythonrc-PROFNAME didn't exist
+            if opts.profile =='':
+                warn("Could not start with profile '%s'!\n"
+                     "('%s/%s.py' does not exist? run '%%upgrade')" %
+                     (opts_all.profile, opts_all.ipythondir, profmodname) )
+        except:
+            print "Error importing",profmodname
+            IP.InteractiveTB()
+            import_fail_info(profmodname)
+                      
+    try:    
+        import ipy_user_conf
+    except ImportError:
+        if opts_all.debug:  IP.InteractiveTB()
+        warn("Could not import user config!\n "
+             "('%s/ipy_user_conf.py' does not exist? Please run '%%upgrade')\n"
+             % opts_all.ipythondir)
+    except:
+        print "Error importing ipy_user_conf"
+        IP.InteractiveTB()
+        import_fail_info("ipy_user_conf")
+    
     # release stdout and stderr and save config log into a global summary
     msg.config.release_all()
-    if IP.rc.messages:
+    if IP_rc.messages:
         msg.summary += msg.config.summary_all()
 
     #------------------------------------------------------------------------
@@ -621,7 +661,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     if load_logplay:
         print 'Replaying log...'
         try:
-            if IP.rc.debug:
+            if IP_rc.debug:
                 logplay_quiet = 0
             else:
                  logplay_quiet = 1
@@ -630,7 +670,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
             IP.safe_execfile(load_logplay,IP.user_ns,
                              islog = 1, quiet = logplay_quiet)
             msg.logplay.release_all()
-            if IP.rc.messages:
+            if IP_rc.messages:
                 msg.summary += msg.logplay.summary_all()
         except:
             warn('Problems replaying logfile %s.' % load_logplay)
@@ -647,89 +687,65 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # ipython is fully initialized, just as if they were run via %run at the
     # ipython prompt.  This would also give them the benefit of ipython's
     # nice tracebacks.
-    
-    if not embedded and IP.rc.args:
+
+    if (not embedded and IP_rc.args and 
+        not IP_rc.args[0].lower().endswith('.ipy')):
         name_save = IP.user_ns['__name__']
         IP.user_ns['__name__'] = '__main__'
+        # Set our own excepthook in case the user code tries to call it
+        # directly. This prevents triggering the IPython crash handler.
+        old_excepthook,sys.excepthook = sys.excepthook, IP.excepthook
+
+        save_argv = sys.argv[1:] # save it for later restoring
+        
+        sys.argv = args
+        
         try:
-            # Set our own excepthook in case the user code tries to call it
-            # directly. This prevents triggering the IPython crash handler.
-            old_excepthook,sys.excepthook = sys.excepthook, IP.excepthook
-            for run in args:
-                IP.safe_execfile(run,IP.user_ns)
+            IP.safe_execfile(args[0], IP.user_ns)
         finally:
             # Reset our crash handler in place
             sys.excepthook = old_excepthook
-            
-        IP.user_ns['__name__'] = name_save
+            sys.argv[:] = save_argv
+            IP.user_ns['__name__'] = name_save
         
     msg.user_exec.release_all()
-    if IP.rc.messages:
+
+    if IP_rc.messages:
         msg.summary += msg.user_exec.summary_all()
 
     # since we can't specify a null string on the cmd line, 0 is the equivalent:
-    if IP.rc.nosep:
-        IP.rc.separate_in = IP.rc.separate_out = IP.rc.separate_out2 = '0'
-    if IP.rc.separate_in == '0': IP.rc.separate_in = ''
-    if IP.rc.separate_out == '0': IP.rc.separate_out = ''
-    if IP.rc.separate_out2 == '0': IP.rc.separate_out2 = ''
-    IP.rc.separate_in = IP.rc.separate_in.replace('\\n','\n')
-    IP.rc.separate_out = IP.rc.separate_out.replace('\\n','\n')
-    IP.rc.separate_out2 = IP.rc.separate_out2.replace('\\n','\n')
+    if IP_rc.nosep:
+        IP_rc.separate_in = IP_rc.separate_out = IP_rc.separate_out2 = '0'
+    if IP_rc.separate_in == '0': IP_rc.separate_in = ''
+    if IP_rc.separate_out == '0': IP_rc.separate_out = ''
+    if IP_rc.separate_out2 == '0': IP_rc.separate_out2 = ''
+    IP_rc.separate_in = IP_rc.separate_in.replace('\\n','\n')
+    IP_rc.separate_out = IP_rc.separate_out.replace('\\n','\n')
+    IP_rc.separate_out2 = IP_rc.separate_out2.replace('\\n','\n')
 
     # Determine how many lines at the bottom of the screen are needed for
     # showing prompts, so we can know wheter long strings are to be printed or
     # paged:
-    num_lines_bot = IP.rc.separate_in.count('\n')+1
-    IP.rc.screen_length = IP.rc.screen_length - num_lines_bot
-    # Initialize cache, set in/out prompts and printing system
-    IP.outputcache = CachedOutput(IP.rc.cache_size,
-                                  IP.rc.pprint,
-                                  input_sep = IP.rc.separate_in,
-                                  output_sep = IP.rc.separate_out,
-                                  output_sep2 = IP.rc.separate_out2,
-                                  ps1 = IP.rc.prompt_in1,
-                                  ps2 = IP.rc.prompt_in2,
-                                  ps_out = IP.rc.prompt_out,
-                                  user_ns = IP.user_ns,
-                                  input_hist = IP.input_hist,
-                                  pad_left = IP.rc.prompts_pad_left)
+    num_lines_bot = IP_rc.separate_in.count('\n')+1
+    IP_rc.screen_length = IP_rc.screen_length - num_lines_bot
 
-    # Set user colors (don't do it in the constructor above so that it doesn't
-    # crash if colors option is invalid)
-    IP.magic_colors(IP.rc.colors)
-    
-    # user may have over-ridden the default print hook:
-    try:
-        IP.outputcache.__class__.display = IP.hooks.display
-    except AttributeError:
-        pass
-
-    # Set calling of pdb on exceptions
-    IP.InteractiveTB.call_pdb = IP.rc.pdb
-    
-    # I don't like assigning globally to sys, because it means when embedding
-    # instances, each embedded instance overrides the previous choice. But
-    # sys.displayhook seems to be called internally by exec, so I don't see a
-    # way around it.
-    sys.displayhook = IP.outputcache
-
-    # we need to know globally if we're caching i/o or not
-    IP.do_full_cache = IP.outputcache.do_full_cache
-    
     # configure startup banner
-    if IP.rc.c:  # regular python doesn't print the banner with -c
-        IP.rc.banner = 0
-    if IP.rc.banner:
-        IP.BANNER = '\n'.join(IP.BANNER_PARTS)
+    if IP_rc.c:  # regular python doesn't print the banner with -c
+        IP_rc.banner = 0
+    if IP_rc.banner:
+        BANN_P = IP.BANNER_PARTS
     else:
-        IP.BANNER = ''
+        BANN_P = []
 
-    if IP.rc.profile: IP.BANNER += '\nIPython profile: '+IP.rc.profile+'\n'
+    if IP_rc.profile: BANN_P.append('IPython profile: %s\n' % IP_rc.profile)
 
     # add message log (possibly empty)
-    IP.BANNER += msg.summary
+    if msg.summary: BANN_P.append(msg.summary)
+    # Final banner is a string
+    IP.BANNER = '\n'.join(BANN_P)
 
+    # Finalize the IPython instance.  This assumes the rc structure is fully
+    # in place.
     IP.post_config_initialization()
 
     return IP
