@@ -60,7 +60,7 @@ You can implement other color schemes easily, the syntax is fairly
 self-explanatory. Please send back new schemes you develop to the author for
 possible inclusion in future releases.
 
-$Id: ultraTB.py,v 1.27 2005/05/30 06:26:28 fperez Exp $"""
+$Id: ultraTB.py 1154 2006-02-11 23:20:05Z fperez $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001 Nathaniel Gray <n8gray@caltech.edu>
@@ -76,21 +76,33 @@ __author__  = '%s <%s>\n%s <%s>' % (Release.authors['Nathan']+
 __license__ = Release.license
 
 # Required modules
-import sys, os, traceback, types, string, time
-import keyword, tokenize, linecache, inspect, pydoc
-from UserDict import UserDict
+import inspect
+import keyword
+import linecache
+import os
+import pydoc
+import string
+import sys
+import time
+import tokenize
+import traceback
+import types
 
 # IPython's own modules
 # Modified pdb which doesn't damage IPython's readline handling
 from IPython import Debugger
-
-from IPython.Struct import Struct
-from IPython.ColorANSI import *
+from IPython.ipstruct import Struct
+from IPython.excolors import ExceptionColors
 from IPython.genutils import Term,uniq_stable,error,info
 
+# Globals
+# amount of space to put line numbers before verbose tracebacks
+INDENT_SIZE = 8
+
 #---------------------------------------------------------------------------
 # Code begins
 
+# Utility functions
 def inspect_error():
     """Print a message about internal inspect errors.
 
@@ -98,128 +110,115 @@ def inspect_error():
     
     error('Internal Python error in the inspect module.\n'
           'Below is the traceback from this internal error.\n')
-    
+
+def _fixed_getinnerframes(etb, context=1,tb_offset=0):
+    import linecache
+    LNUM_POS, LINES_POS, INDEX_POS =  2, 4, 5
+
+    records  = inspect.getinnerframes(etb, context)
+
+    # If the error is at the console, don't build any context, since it would
+    # otherwise produce 5 blank lines printed out (there is no file at the
+    # console)
+    rec_check = records[tb_offset:]
+    try:
+        rname = rec_check[0][1]
+        if rname == '<ipython console>' or rname.endswith('<string>'):
+            return rec_check
+    except IndexError:
+        pass
+
+    aux = traceback.extract_tb(etb)
+    assert len(records) == len(aux)
+    for i, (file, lnum, _, _) in zip(range(len(records)), aux):
+        maybeStart = lnum-1 - context//2
+        start =  max(maybeStart, 0)
+        end   = start + context
+        lines = linecache.getlines(file)[start:end]
+        # pad with empty lines if necessary
+        if maybeStart < 0:
+            lines = (['\n'] * -maybeStart) + lines
+        if len(lines) < context:
+            lines += ['\n'] * (context - len(lines))
+        buf = list(records[i])
+        buf[LNUM_POS] = lnum
+        buf[INDEX_POS] = lnum - 1 - start
+        buf[LINES_POS] = lines
+        records[i] = tuple(buf)
+    return records[tb_offset:]
+
+# Helper function -- largely belongs to VerboseTB, but we need the same
+# functionality to produce a pseudo verbose TB for SyntaxErrors, so that they
+# can be recognized properly by ipython.el's py-traceback-line-re
+# (SyntaxErrors have to be treated specially because they have no traceback)
+def _formatTracebackLines(lnum, index, lines, Colors, lvals=None):
+    numbers_width = INDENT_SIZE - 1
+    res = []
+    i = lnum - index
+    for line in lines:
+        if i == lnum:
+            # This is the line with the error
+            pad = numbers_width - len(str(i))
+            if pad >= 3:
+                marker = '-'*(pad-3) + '-> '
+            elif pad == 2:
+                marker = '> '
+            elif pad == 1:
+                marker = '>'
+            else:
+                marker = ''
+            num = marker + str(i)
+            line = '%s%s%s %s%s' %(Colors.linenoEm, num, 
+                                   Colors.line, line, Colors.Normal)
+        else:
+            num = '%*s' % (numbers_width,i)
+            line = '%s%s%s %s' %(Colors.lineno, num, 
+                                 Colors.Normal, line)
+
+        res.append(line)
+        if lvals and i == lnum:
+            res.append(lvals + '\n')
+        i = i + 1
+    return res
+
+#---------------------------------------------------------------------------
+# Module classes
 class TBTools:
     """Basic tools used by all traceback printer classes."""
 
-    def __init__(self,color_scheme = 'NoColor',call_pdb=0):
+    def __init__(self,color_scheme = 'NoColor',call_pdb=False):
         # Whether to call the interactive pdb debugger after printing
         # tracebacks or not
         self.call_pdb = call_pdb
-        if call_pdb:
-            self.pdb = Debugger.Pdb()
-        else:
-            self.pdb = None
 
         # Create color table
-        self.ColorSchemeTable = ColorSchemeTable()
-        
-        # Populate it with color schemes
-        C = TermColors # shorthand and local lookup
-        self.ColorSchemeTable.add_scheme(ColorScheme(
-            'NoColor',
-            # The color to be used for the top line
-            topline = C.NoColor,
-
-            # The colors to be used in the traceback
-            filename = C.NoColor,
-            lineno = C.NoColor,
-            name = C.NoColor,
-            vName = C.NoColor,
-            val = C.NoColor,
-            em = C.NoColor,
-
-            # Emphasized colors for the last frame of the traceback
-            normalEm = C.NoColor,
-            filenameEm = C.NoColor,
-            linenoEm = C.NoColor,
-            nameEm = C.NoColor,
-            valEm = C.NoColor,
-
-            # Colors for printing the exception
-            excName = C.NoColor,
-            line = C.NoColor,
-            caret = C.NoColor,
-            Normal = C.NoColor
-            ))
-
-        # make some schemes as instances so we can copy them for modification easily:
-        self.ColorSchemeTable.add_scheme(ColorScheme(
-            'Linux',
-            # The color to be used for the top line
-            topline = C.LightRed,
-
-            # The colors to be used in the traceback
-            filename = C.Green,
-            lineno = C.Green,
-            name = C.Purple,
-            vName = C.Cyan,
-            val = C.Green,
-            em = C.LightCyan,
-
-            # Emphasized colors for the last frame of the traceback
-            normalEm = C.LightCyan,
-            filenameEm = C.LightGreen,
-            linenoEm = C.LightGreen,
-            nameEm = C.LightPurple,
-            valEm = C.LightBlue,
-
-            # Colors for printing the exception
-            excName = C.LightRed,
-            line = C.Yellow,
-            caret = C.White,
-            Normal = C.Normal
-            ))
-
-        # For light backgrounds, swap dark/light colors
-        self.ColorSchemeTable.add_scheme(ColorScheme(
-            'LightBG',
-            # The color to be used for the top line
-            topline = C.Red,
-
-            # The colors to be used in the traceback
-            filename = C.LightGreen,
-            lineno = C.LightGreen,
-            name = C.LightPurple,
-            vName = C.Cyan,
-            val = C.LightGreen,
-            em = C.Cyan,
-
-            # Emphasized colors for the last frame of the traceback
-            normalEm = C.Cyan,
-            filenameEm = C.Green,
-            linenoEm = C.Green,
-            nameEm = C.Purple,
-            valEm = C.Blue,
-
-            # Colors for printing the exception
-            excName = C.Red,
-            #line = C.Brown,  # brown often is displayed as yellow
-            line = C.Red,
-            caret = C.Normal,
-            Normal = C.Normal
-            ))
+        self.color_scheme_table = ExceptionColors 
 
         self.set_colors(color_scheme)
         self.old_scheme = color_scheme  # save initial value for toggles
 
+        if call_pdb:
+            self.pdb = Debugger.Pdb(self.color_scheme_table.active_scheme_name)
+        else:
+            self.pdb = None
+
     def set_colors(self,*args,**kw):
         """Shorthand access to the color table scheme selector method."""
         
-        self.ColorSchemeTable.set_active_scheme(*args,**kw)
+        self.color_scheme_table.set_active_scheme(*args,**kw)
         # for convenience, set Colors to the active scheme
-        self.Colors = self.ColorSchemeTable.active_colors
+        self.Colors = self.color_scheme_table.active_colors
 
     def color_toggle(self):
         """Toggle between the currently active color scheme and NoColor."""
         
-        if self.ColorSchemeTable.active_scheme_name == 'NoColor':
-            self.ColorSchemeTable.set_active_scheme(self.old_scheme)
-            self.Colors = self.ColorSchemeTable.active_colors
+        if self.color_scheme_table.active_scheme_name == 'NoColor':
+            self.color_scheme_table.set_active_scheme(self.old_scheme)
+            self.Colors = self.color_scheme_table.active_colors
         else:
-            self.old_scheme = self.ColorSchemeTable.active_scheme_name
-            self.ColorSchemeTable.set_active_scheme('NoColor')
-            self.Colors = self.ColorSchemeTable.active_colors
+            self.old_scheme = self.color_scheme_table.active_scheme_name
+            self.color_scheme_table.set_active_scheme('NoColor')
+            self.Colors = self.color_scheme_table.active_colors
 
 #---------------------------------------------------------------------------
 class ListTB(TBTools):
@@ -303,12 +302,12 @@ class ListTB(TBTools):
         """Format the exception part of a traceback.
 
         The arguments are the exception type and value such as given by
-        sys.last_type and sys.last_value. The return value is a list of
-        strings, each ending in a newline.  Normally, the list contains a
-        single string; however, for SyntaxError exceptions, it contains
-        several lines that (when printed) display detailed information
-        about where the syntax error occurred.  The message indicating
-        which exception occurred is the always last string in the list.
+        sys.exc_info()[:2]. The return value is a list of strings, each ending
+        in a newline.  Normally, the list contains a single string; however,
+        for SyntaxError exceptions, it contains several lines that (when
+        printed) display detailed information about where the syntax error
+        occurred.  The message indicating which exception occurred is the
+        always last string in the list.
         
         Also lifted nearly verbatim from traceback.py
         """
@@ -394,15 +393,39 @@ class VerboseTB(TBTools):
         # some locals
         Colors        = self.Colors   # just a shorthand + quicker name lookup
         ColorsNormal  = Colors.Normal  # used a lot
-        indent_size   = 8  # we need some space to put line numbers before
-        indent        = ' '*indent_size
-        numbers_width = indent_size - 1 # leave space between numbers & code
-        text_repr     = pydoc.text.repr
+        indent        = ' '*INDENT_SIZE
         exc           = '%s%s%s' % (Colors.excName, str(etype), ColorsNormal)
         em_normal     = '%s\n%s%s' % (Colors.valEm, indent,ColorsNormal)
         undefined     = '%sundefined%s' % (Colors.em, ColorsNormal)
 
         # some internal-use functions
+        def text_repr(value):
+            """Hopefully pretty robust repr equivalent."""
+            # this is pretty horrible but should always return *something*
+            try:
+                return pydoc.text.repr(value)
+            except KeyboardInterrupt:
+                raise
+            except:
+                try:
+                    return repr(value)
+                except KeyboardInterrupt:
+                    raise
+                except:
+                    try:
+                        # all still in an except block so we catch
+                        # getattr raising
+                        name = getattr(value, '__name__', None)
+                        if name:
+                            # ick, recursion
+                            return text_repr(name)
+                        klass = getattr(value, '__class__', None)
+                        if klass:
+                            return '%s instance' % text_repr(klass)
+                    except KeyboardInterrupt:
+                        raise
+                    except:
+                        return 'UNRECOVERABLE REPR FAILURE'
         def eqrepr(value, repr=text_repr): return '=%s' % repr(value)
         def nullrepr(value, repr=text_repr): return ''
 
@@ -431,7 +454,13 @@ class VerboseTB(TBTools):
         linecache.checkcache()
         # Drop topmost frames if requested
         try:
-            records = inspect.getinnerframes(etb, context)[self.tb_offset:]
+            # Try the default getinnerframes and Alex's: Alex's fixes some
+            # problems, but it generates empty tracebacks for console errors
+            # (5 blanks lines) where none should be returned.
+            #records = inspect.getinnerframes(etb, context)[self.tb_offset:]
+            #print 'python records:', records # dbg
+            records = _fixed_getinnerframes(etb, context,self.tb_offset)
+            #print 'alex   records:', records # dbg
         except:
 
             # FIXME: I've been getting many crash reports from python 2.3
@@ -597,32 +626,12 @@ class VerboseTB(TBTools):
                 lvals = ''
 
             level = '%s %s\n' % (link,call)
-            excerpt = []
-            if index is not None:
-                i = lnum - index
-                for line in lines:
-                    if i == lnum:
-                        # This is the line with the error
-                        pad = numbers_width - len(str(i))
-                        if pad >= 3:
-                            marker = '-'*(pad-3) + '-> '
-                        elif pad == 2:
-                            marker = '> '
-                        elif pad == 1:
-                            marker = '>'
-                        else:
-                            marker = ''
-                        num = '%s%s' % (marker,i)
-                        line = tpl_line_em % (num,line)
-                    else:
-                        num = '%*s' % (numbers_width,i)
-                        line = tpl_line % (num,line)
 
-                    excerpt.append(line)
-                    if self.include_vars and i == lnum:
-                        excerpt.append('%s\n' % lvals)
-                    i += 1
-            frames.append('%s%s' % (level,''.join(excerpt)) )
+            if index is None:
+                frames.append(level)
+            else:
+                frames.append('%s%s' % (level,''.join(
+                    _formatTracebackLines(lnum,index,lines,self.Colors,lvals))))
 
         # Get (safely) a string form of the exception info
         try:
@@ -635,7 +644,19 @@ class VerboseTB(TBTools):
         exception = ['%s%s%s: %s' % (Colors.excName, etype_str,
                                      ColorsNormal, evalue_str)]
         if type(evalue) is types.InstanceType:
-            for name in dir(evalue):
+            try:
+                names = [w for w in dir(evalue) if isinstance(w, basestring)]
+            except:
+                # Every now and then, an object with funny inernals blows up
+                # when dir() is called on it.  We do the best we can to report
+                # the problem and continue
+                _m = '%sException reporting error (object with broken dir())%s:'
+                exception.append(_m % (Colors.excName,ColorsNormal))
+                etype_str,evalue_str = map(str,sys.exc_info()[:2])
+                exception.append('%s%s%s: %s' % (Colors.excName,etype_str,
+                                     ColorsNormal, evalue_str))
+                names = []
+            for name in names:
                 value = text_repr(getattr(evalue, name))
                 exception.append('\n%s%s = %s' % (indent, name, value))
         # return all our info assembled as a single string
@@ -655,14 +676,21 @@ class VerboseTB(TBTools):
 
         if self.call_pdb:
             if self.pdb is None:
-                self.pdb = Debugger.Pdb()
-            # the system displayhook may have changed, restore the original for pdb
+                self.pdb = Debugger.Pdb(
+                    self.color_scheme_table.active_scheme_name)
+            # the system displayhook may have changed, restore the original
+            # for pdb
             dhook = sys.displayhook
             sys.displayhook = sys.__displayhook__
             self.pdb.reset()
+            # Find the right frame so we don't pop up inside ipython itself
+            etb = self.tb
             while self.tb.tb_next is not None:
                 self.tb = self.tb.tb_next
             try:
+                if etb and etb.tb_next:
+                    etb = etb.tb_next
+                self.pdb.botframe = etb.tb_frame
                 self.pdb.interaction(self.tb.tb_frame, self.tb)
             except:
                 print '*** ERROR ***'
@@ -680,10 +708,10 @@ class VerboseTB(TBTools):
     # out the right info on its own.
     def __call__(self, etype=None, evalue=None, etb=None):
         """This hook can replace sys.excepthook (for Python 2.1 or higher)."""
-        if etb is not None:
-            self.handler((etype, evalue, etb))
-        else:
+        if etb is None:
             self.handler()
+        else:
+            self.handler((etype, evalue, etb))
         self.debugger()
 
 #----------------------------------------------------------------------------
@@ -796,7 +824,7 @@ class AutoFormattedTB(FormattedTB):
             print >> out, self.text(etype, evalue, etb)
             self.tb_offset = tb_offset
         else:
-            print >> out, self.text()
+            print >> out, self.text(etype, evalue, etb)
         self.debugger()
 
     def text(self,etype=None,value=None,tb=None,context=5,mode=None):
