@@ -6,7 +6,7 @@
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-# $Id: usage.py,v 1.22 2005/05/13 21:16:51 fperez Exp $
+# $Id: usage.py 1332 2006-05-30 01:41:28Z fperez $
 
 from IPython import Release
 __author__  = '%s <%s>' % Release.authors['Fernando']
@@ -67,6 +67,11 @@ SPECIAL THREADING OPTIONS
               essentially the same functionality, respectively for GTK, QT and
               WXWidgets (via their Python interfaces).
 
+              Note that with -wthread, you can additionally use the -wxversion
+              option to request a specific version of wx to be used.  This
+              requires that you have the 'wxversion' Python module installed,
+              which is part of recent wxPython distributions.
+
               If -pylab is given, IPython loads special support for the mat-
               plotlib library (http://matplotlib.sourceforge.net), allowing
               interactive usage of any of its backends as defined in the
@@ -128,8 +133,8 @@ REGULAR OPTIONS
        See the provided examples for assistance.  Options given on the comman-
        dline override the values set in the ipythonrc file.
 
-       All options with a no| prepended can be specified in ’no’ form  (-noop-
-       tion instead of -option) to turn the feature off.
+       All options with a [no] prepended can be specified in negated form
+       (using -nooption instead of -option) to turn the feature off.
 
        -h, --help
               Show summary of options.
@@ -138,35 +143,38 @@ REGULAR OPTIONS
               will have no effect in any other position). It adds special sup-
               port   for  the  matplotlib  library  (http://matplotlib.source-
               forge.net), allowing interactive usage of any of its backends as
-              defined  in  the  user’s  .matplotlibrc  file.  It automatically
+              defined  in  the  user's  .matplotlibrc  file.  It automatically
               activates GTK or WX threading for IPyhton if the choice of  mat-
               plotlib  backend requires it.  It also modifies the @run command
               to correctly execute  (without  blocking)  any  matplotlib-based
               script which calls show() at the end.
 
-       -no|autocall
-              Make  IPython automatically call any callable object even if you
-              didn’t type explicit parentheses. For example, ’str 43’  becomes
-              ’str(43)’ automatically.
+       -autocall <val>
+              Make IPython automatically call any callable object even if you
+              didn't type explicit parentheses. For example, 'str 43' becomes
+              'str(43)' automatically.  The value can be '0' to disable the
+              feature, '1' for 'smart' autocall, where it is not applied if
+              there are no more arguments on the line, and '2' for 'full'
+              autocall, where all callable objects are automatically called
+              (even if no arguments are present).  The default is '1'.
 
-       -no|autoindent
+       -[no]autoindent
               Turn automatic indentation on/off.
 
-       -no|automagic
+       -[no]automagic
               Make magic commands automatic (without needing their first char-
-              acter to be @).  Type @magic at  the  IPython  prompt  for  more
+              acter to be %).  Type %magic at  the  IPython  prompt  for  more
               information.
 
-       -no|autoparens
-              Make  IPython automatically call any callable object even if you
-              didn’t type explicit parentheses.  For example, ’str 43’ becomes
-              ’str(43)’ automatically.
+       -[no]autoedit_syntax
+              When a syntax error occurs after editing a file, automatically
+              open the file to the trouble causing line for convenient fixing.
 
-       -no|banner
+       -[no]banner
               Print the intial information banner (default on).
 
        -c <command>
-              Execute  the  given  command  string, and set sys.argv to [’c’].
+              Execute  the  given  command  string, and set sys.argv to ['c'].
               This is similar to the -c option in  the  normal  Python  inter-
               preter.
 
@@ -176,7 +184,7 @@ REGULAR OPTIONS
               your  config  file.   Setting  it  to  0 completely disables the
               caching system, and the minimum value accepted  is  20  (if  you
               provide  a value less than 20, it is reset to 0 and a warning is
-              issued).  This limit is defined because otherwise  you’ll  spend
+              issued).  This limit is defined because otherwise  you'll  spend
               more time re-flushing a too small cache than working.
 
        -classic|cl
@@ -186,36 +194,36 @@ REGULAR OPTIONS
               Color  scheme  for  prompts  and exception reporting.  Currently
               implemented: NoColor, Linux, and LightBG.
 
-       -no|color_info
+       -[no]color_info
               IPython can display information about objects via a set of func-
               tions, and optionally can use colors for this, syntax highlight-
               ing source code and various other  elements.   However,  because
-              this  information  is  passed  through a pager (like ’less’) and
+              this  information  is  passed  through a pager (like 'less') and
               many pagers get confused with color codes, this option is off by
               default.   You  can  test  it and turn it on permanently in your
-              ipythonrc file if it works for you.  As a reference, the  ’less’
+              ipythonrc file if it works for you.  As a reference, the  'less'
               pager  supplied  with  Mandrake 8.2 works ok, but that in RedHat
-              7.2 doesn’t.
+              7.2 doesn't.
 
               Test it and turn it on permanently if it works with your system.
               The  magic function @color_info allows you to toggle this inter-
               actively for testing.
 
-       -no|confirm_exit
+       -[no]confirm_exit
               Set to confirm when you try to exit IPython with  an  EOF  (Con-
               trol-D in Unix, Control-Z/Enter in Windows). Note that using the
               magic functions @Exit or @Quit you  can  force  a  direct  exit,
               bypassing any confirmation.
 
-       -no|debug
+       -[no]debug
               Show  information  about the loading process. Very useful to pin
               down problems with your configuration files or  to  get  details
               about session restores.
 
-       -no|deep_reload
+       -[no]deep_reload
               IPython  can use the deep_reload module which reloads changes in
               modules recursively (it replaces the reload() function,  so  you
-              don’t need to change anything to use it). deep_reload() forces a
+              don't need to change anything to use it). deep_reload() forces a
               full reload of modules whose code may have  changed,  which  the
               default reload() function does not.
 
@@ -237,46 +245,46 @@ REGULAR OPTIONS
               This can also be  specified  through  the  environment  variable
               IPYTHONDIR.
 
-       -log|l Generate  a log file of all input. The file is named ipython.log
-              in your current directory (which  prevents  logs  from  multiple
-              IPython sessions from trampling each other). You can use this to
-              later restore a session by loading your logfile as a file to  be
-              executed with option -logplay (see below).
+       -log|l Generate  a log file of all input. The file is named
+              ipython_log.py in your current directory (which prevents logs
+              from multiple IPython sessions from trampling each other). You
+              can use this to later restore a session by loading your logfile
+              as a file to be executed with option -logplay (see below).
 
        -logfile|lf
-              Specifu the name of your logfile.
+              Specify the name of your logfile.
 
        -logplay|lp
               Replay  a previous log. For restoring a session as close as pos-
-              sible to the state you left it in, use this option  (don’t  just
+              sible to the state you left it in, use this option  (don't  just
               run the logfile). With -logplay, IPython will try to reconstruct
               the previous working environment in full, not just  execute  the
               commands in the logfile.
               When  a  session is restored, logging is automatically turned on
               again with the name of the logfile it was invoked  with  (it  is
-              read  from the log header). So once you’ve turned logging on for
+              read  from the log header). So once you've turned logging on for
               a session, you can quit IPython and reload it as many  times  as
               you  want  and  it  will continue to log its history and restore
               from the beginning every time.
 
               Caveats: there are limitations in this option. The history vari-
-              ables  _i*,_* and _dh don’t get restored properly. In the future
+              ables  _i*,_* and _dh don't get restored properly. In the future
               we will try to implement full  session  saving  by  writing  and
-              retrieving  a failed because of inherent limitations of Python’s
+              retrieving  a failed because of inherent limitations of Python's
               Pickle module, so this may have to wait.
 
-       -no|messages
+       -[no]messages
               Print messages which IPython collects about its startup  process
               (default on).
 
-       -no|pdb
+       -[no]pdb
               Automatically  call the pdb debugger after every uncaught excep-
               tion. If you are used to debugging  using  pdb,  this  puts  you
               automatically  inside of it after any call (either in IPython or
               in code called by it) which triggers  an  exception  which  goes
               uncaught.
 
-       -no|pprint
+       -[no]pprint
               IPython  can  optionally  use the pprint (pretty printer) module
               for displaying results. pprint tends to give a nicer display  of
               nested  data structures. If you like it, you can turn it on per-
@@ -287,7 +295,7 @@ REGULAR OPTIONS
               rent dir first, then in IPYTHONDIR). This is a quick way to keep
               and load multiple config files for different  tasks,  especially
               if  you  use  the include option of config files. You can keep a
-              basic IPYTHONDIR/ipythonrc file and then have  other  ’profiles’
+              basic IPYTHONDIR/ipythonrc file and then have  other  'profiles'
               which  include  this  one  and  load extra things for particular
               tasks. For example:
 
@@ -303,25 +311,25 @@ REGULAR OPTIONS
 
        -prompt_in1|pi1 <string>
               Specify  the string used for input prompts. Note that if you are
-              using numbered prompts, the number is represented with a ’\#’ in
-              the  string.  Don’t forget to quote strings with spaces embedded
-              in them. Default: ’In [\#]:’.
+              using numbered prompts, the number is represented with a '\#' in
+              the  string.  Don't forget to quote strings with spaces embedded
+              in them. Default: 'In [\#]: '.
 
-              Most bash-like  escapes  can  be  used  to  customize  IPython’s
+              Most bash-like  escapes  can  be  used  to  customize  IPython's
               prompts, as well as a few additional ones which are IPython-spe-
               cific.  All valid prompt escapes are described in detail in  the
               Customization section of the IPython HTML/PDF manual.
 
        -prompt_in2|pi2 <string>
-              Similar  to  the  previous option, but used for the continuation
-              prompts. The special sequence ’\D’ is similar to ’\#’, but  with
-              all  digits  replaced  dots  (so  you can have your continuation
-              prompt aligned with your  input  prompt).  Default:  ’    .\D.:’
-              (note three spaces at the start for alignment with ’In [\#]’).
+              Similar to the previous option, but used for the continuation
+              prompts. The special sequence '\D' is similar to '\#', but with
+              all digits replaced dots (so you can have your continuation
+              prompt aligned with your input prompt).  Default: ' .\D.: '
+              (note three spaces at the start for alignment with 'In [\#]').
 
        -prompt_out|po <string>
               String   used   for  output  prompts,  also  uses  numbers  like
-              prompt_in1.  Default: ’Out[\#]:’.
+              prompt_in1.  Default: 'Out[\#]:'.
 
        -quick Start in bare bones mode (no config file loaded).
 
@@ -332,14 +340,14 @@ REGULAR OPTIONS
               IPython  starts  with  a  bare  bones  configuration (no modules
               loaded at all).
 
-       -no|readline
+       -[no]readline
               Use the readline library, which is needed to support  name  com-
               pletion  and  command history, among other things. It is enabled
               by default, but may cause  problems  for  users  of  X/Emacs  in
               Python comint or shell buffers.
 
-              Note  that  emacs ’eterm’ buffers (opened with M-x term) support
-              IPython’s readline and syntax coloring fine, only  ’emacs’  (M-x
+              Note  that  emacs 'eterm' buffers (opened with M-x term) support
+              IPython's readline and syntax coloring fine, only  'emacs'  (M-x
               shell and C-c !)  buffers do not.
 
        -screen_length|sl <n>
@@ -349,13 +357,13 @@ REGULAR OPTIONS
 
               The default value for this is 0, which means IPython will  auto-
               detect  your  screen  size  every time it needs to print certain
-              potentially long strings (this doesn’t change  the  behavior  of
-              the  ’print’  keyword,  it’s  only triggered internally). If for
-              some reason this isn’t working well (it needs  curses  support),
-              specify it yourself. Otherwise don’t change the default.
+              potentially long strings (this doesn't change  the  behavior  of
+              the  'print'  keyword,  it's  only triggered internally). If for
+              some reason this isn't working well (it needs  curses  support),
+              specify it yourself. Otherwise don't change the default.
 
        -separate_in|si <string>
-              Separator before input prompts.  Default ’0.
+              Separator before input prompts.  Default '0.
 
        -separate_out|so <string>
               Separator before output prompts.  Default: 0 (nothing).
@@ -363,7 +371,7 @@ REGULAR OPTIONS
        -separate_out2|so2 <string>
               Separator after output prompts.  Default: 0 (nothing).
 
-       -nosep Shorthand for ’-separate_in 0 -separate_out 0 -separate_out2 0’.
+       -nosep Shorthand for '-separate_in 0 -separate_out 0 -separate_out2 0'.
               Simply removes all input/output separators.
 
        -upgrade
@@ -377,11 +385,16 @@ REGULAR OPTIONS
        -Version
               Print version information and exit.
 
+       -wxversion <string>
+              Select a specific version of wxPython (used in conjunction with
+              -wthread). Requires the wxversion module, part of recent
+              wxPython distributions.
+
        -xmode <modename>
               Mode  for  exception reporting.  The valid modes are Plain, Con-
               text, and Verbose.
 
-              - Plain: similar to python’s normal traceback printing.
+              - Plain: similar to python's normal traceback printing.
 
               - Context: prints 5 lines of context  source  code  around  each
               line in the traceback.
@@ -405,7 +418,7 @@ EMBEDDING
        code,  operate  with  your  variables, analyze them, etc.  Note however
        that any changes you make to values while in the shell do NOT propagate
        back  to  the running code, so it is safe to modify your values because
-       you won’t break your code in bizarre ways by doing so.
+       you won't break your code in bizarre ways by doing so.
 """
 
 cmd_line_usage = __doc__
@@ -584,3 +597,45 @@ MAIN FEATURES
         won't work:
             >>> x = ,my_function /home/me    # syntax error
 """
+
+quick_reference = r"""
+IPython -- An enhanced Interactive Python - Quick Reference Card
+================================================================
+
+obj?, obj??, ?obj,??obj   : Get help, or more help for object
+?os.p*           : List names in os starting with p
+
+Example magic:
+    
+%alias d ls -F   : 'd' is now an alias for 'ls -F'
+alias d ls -F    : Works if 'alias' not a python name
+alist = %alias   : Get list of aliases to 'alist'
+
+System commands:
+    
+!cp a.txt b/     : System command escape, calls os.system()
+cp a.txt b/      : after %rehashx, most system commands work without !
+cp ${f}.txt $bar : Variable expansion in magics and system commands
+files = !ls /usr : Capture sytem command output
+files.s, files.l, files.n: "a b c", ['a','b','c'], 'a\nb\nc'
+cd /usr/share    : Obvious, also 'cd d:\home\_ipython' works
+
+History:
+
+_i, _ii, _iii    : Previous, next previous, next next previous input
+_ih[4], _ih[2:5] : Input history line 4, lines 2-4
+_, __, ___       : previous, next previous, next next previous output
+_dh              : Directory history
+_oh              : Output history
+%hist            : Command history
+
+Autocall:
+
+f 1,2            : f(1,2)
+/f 1,2           : f(1,2) (forced autoparen)
+,f 1 2           : f("1","2")
+;f 1 2           : f("1 2")
+
+"""
+
+
