@@ -6,7 +6,7 @@
 ;; URL:           http://ipython.scipy.org
 ;; Compatibility: Emacs21, XEmacs21
 ;; FIXME: #$@! INPUT RING
-(defconst ipython-version "$Revision: 1.3 $"
+(defconst ipython-version "$Revision: 1325 $"
   "VC version number.")
 
 ;;; Commentary 
@@ -47,8 +47,8 @@
 ;; -------------------------
 ;;
 ;; - IMO the best feature by far of the ipython/emacs combo is how much easier it
-;;   makes it to find and fix bugs thanks to the ``@pdb on``/ pdbtrack combo. Try
-;;   it: first in the ipython to shell do ``@pdb on`` then do something that will
+;;   makes it to find and fix bugs thanks to the ``%pdb on``/ pdbtrack combo. Try
+;;   it: first in the ipython to shell do ``%pdb on`` then do something that will
 ;;   raise an exception (FIXME nice example) -- and be amazed how easy it is to
 ;;   inspect the live objects in each stack frames and to jump to the
 ;;   corresponding sourcecode locations as you walk up and down the stack trace
@@ -177,7 +177,9 @@ the second for a 'normal' command, and the third for a multiline command.")
       (define-key py-shell-map [tab] 'ipython-complete)
       ;;XXX this is really just a cheap hack, it only completes symbols in the
       ;;interactive session -- useful nonetheless.
-      (define-key py-mode-map [(meta tab)] 'ipython-complete))
+      (define-key py-mode-map [(meta tab)] 'ipython-complete)
+      
+      )
     (add-hook 'py-shell-hook 'ipython-shell-hook)
     ;; Regular expression that describes tracebacks for IPython in context and
     ;; verbose mode. 
@@ -191,9 +193,13 @@ the second for a 'normal' command, and the third for a multiline command.")
     ;;     (XXX: should ask Fernando for a change)
     ;;"^   File \"\\(.*?\\)\", line \\([0-9]+\\).*\n.*\n.*\nSyntaxError:"
     ;;^   File \"\\(.*?\\)\", line \\([0-9]+\\)"
+
     (setq py-traceback-line-re
           "\\(^[^\t ].+?\\.py\\).*\n   +[0-9]+[^\00]*?\n-+> \\([0-9]+\\) +")
-
+    
+    ;; Recognize the ipython pdb, whose prompt is 'ipdb>' instead of '(Pdb)'
+    (setq py-pdbtrack-input-prompt "\n[(<]*[Ii]?[Pp]db[>)]+ ")
+    
     (setq py-shell-input-prompt-1-regexp "^In \\[[0-9]+\\]: *"
           py-shell-input-prompt-2-regexp "^   [.][.][.]+: *" )
     ;; select a suitable color-scheme
@@ -305,6 +311,9 @@ in the current *Python* session."
       (interactive)
       (let* ((ugly-return nil)
              (sep ";")
+             (python-process (or (get-buffer-process (current-buffer))
+                                 ;XXX hack for .py buffers
+                                 (get-process py-which-bufname)))
              ;; XXX currently we go backwards to find the beginning of an
              ;; expression part; a more powerful approach in the future might be
              ;; to let ipython have the complete line, so that context can be used
@@ -325,10 +334,9 @@ in the current *Python* session."
                           (delete-region comint-last-output-start 
                                          (process-mark (get-buffer-process (current-buffer)))))))))
                                         ;(message (format "#DEBUG pattern: '%s'" pattern))
-        (process-send-string  (or (get-buffer-process (current-buffer))
-                                  (get-process py-which-bufname)) ;XXX hack for .py buffers
+        (process-send-string python-process 
                               (format ipython-completion-command-string pattern))
-        (accept-process-output (get-buffer-process (current-buffer)))
+        (accept-process-output python-process)
                                         ;(message (format "DEBUG return: %s" ugly-return))
         (setq completions 
               (split-string (substring ugly-return 0 (position ?\n ugly-return)) sep))
@@ -354,6 +362,9 @@ in the current *Python* session."
     (interactive)
     (let* ((ugly-return nil)
            (sep ";")
+           (python-process (or (get-buffer-process (current-buffer))
+                                        ;XXX hack for .py buffers
+                               (get-process py-which-bufname)))
            ;; XXX currently we go backwards to find the beginning of an
            ;; expression part; a more powerful approach in the future might be
            ;; to let ipython have the complete line, so that context can be used
@@ -371,10 +382,9 @@ in the current *Python* session."
                     (lambda (string) 
                       (setq ugly-return (concat ugly-return string))
                       "")))))
-      (process-send-string  (or (get-buffer-process (current-buffer))
-                                (get-process py-which-bufname)) ;XXX hack for .py buffers
+      (process-send-string python-process 
                             (format ipython-completion-command-string pattern))
-      (accept-process-output (get-buffer-process (current-buffer)))
+      (accept-process-output python-process)
       (setq completions 
             (split-string (substring ugly-return 0 (position ?\n ugly-return)) sep))
                                         ;(message (format "DEBUG completions: %S" completions))
@@ -395,4 +405,81 @@ in the current *Python* session."
              (message "Making completion list...%s" "done")))))
 )
 
+;;; autoindent support: patch sent in by Jin Liu <m.liu.jin@gmail.com>,
+;;; originally written by doxgen@newsmth.net
+;;; Minor modifications by fperez for xemacs compatibility.
+
+(defvar ipython-autoindent t
+ "If non-nil, enable autoindent for IPython shell through python-mode.")
+
+(defvar ipython-indenting-buffer-name "*IPython Indentation Calculation*"
+ "Temporary buffer for indenting multiline statement.")
+
+(defun ipython-get-indenting-buffer ()
+ "Return a temporary buffer set in python-mode. Create one if necessary."
+ (let ((buf (get-buffer-create ipython-indenting-buffer-name)))
+   (set-buffer buf)
+   (unless (eq major-mode 'python-mode)
+     (python-mode))
+   buf))
+
+(defvar ipython-indentation-string nil
+ "Indentation for the next line in a multiline statement.")
+
+(defun ipython-send-and-indent ()
+ "Send the current line to IPython, and calculate the indentation for
+the next line."
+ (interactive)
+ (if ipython-autoindent
+     (let ((line (buffer-substring (point-at-bol) (point)))
+           (after-prompt1)
+           (after-prompt2))
+       (save-excursion
+           (comint-bol t)
+           (if (looking-at py-shell-input-prompt-1-regexp)
+               (setq after-prompt1 t)
+             (setq after-prompt2 (looking-at py-shell-input-prompt-2-regexp)))
+           (with-current-buffer (ipython-get-indenting-buffer)
+             (when after-prompt1
+               (erase-buffer))
+             (when (or after-prompt1 after-prompt2)
+               (delete-region (point-at-bol) (point))
+               (insert line)
+               (newline-and-indent))))))
+ ;; send input line to ipython interpreter
+ (comint-send-input))
+
+(defun ipython-indentation-hook (string)
+ "Insert indentation string if py-shell-input-prompt-2-regexp
+matches last process output."
+ (let* ((start-marker (or comint-last-output-start
+                          (point-min-marker)))
+        (end-marker (process-mark (get-buffer-process (current-buffer))))
+        (text (ansi-color-filter-apply (buffer-substring start-marker end-marker))))
+   (progn
+     ;; XXX if `text' matches both pattern, it MUST be the last prompt-2
+     (cond ((and (string-match "\n$" text) (string-match
+py-shell-input-prompt-2-regexp text))
+            (with-current-buffer (ipython-get-indenting-buffer)
+              (erase-buffer)))
+           ;; still a prompt-2
+           ((string-match py-shell-input-prompt-2-regexp text)
+            (with-current-buffer (ipython-get-indenting-buffer)
+              (setq ipython-indentation-string
+                    (buffer-substring (point-at-bol) (point))))
+            (unless (eq ipython-indentation-string nil)
+              (message "ipython-indentation-hook: %s#%s##" text
+ipython-indentation-string))
+            (goto-char end-marker)
+            (insert ipython-indentation-string)))
+           (setq ipython-indentation-string nil))))
+
+(add-hook 'py-shell-hook
+         (lambda ()
+           (add-hook 'comint-output-filter-functions
+                     'ipython-indentation-hook)))
+
+(define-key py-shell-map (kbd "RET") 'ipython-send-and-indent)
+;;; / end autoindent support
+
 (provide 'ipython)
